<?php

namespace Model;

require_once CLASS_PATH.'/table/d_table.php';
require_once CLASS_PATH.'/table/p_table.php';

class base
{

// Global meta-class

protected static $__db;

protected static $__logger;

protected static $_cache = [];

// Notification
protected static $_notif_email = 'mathieu@iprospective.fr';

// Class
protected static $_id;
protected static $_name;
protected static $_label;

// Tables
protected static $_pt = [];
protected static $_pt_alias = [];
protected static $_dt = [];
protected static $_dt_alias = [];

// Params
protected static $__p_params = [
	'id_lang' => P_LANG_ID,
	'id_shop' => P_SHOP_ID,
	'lang' => 'fr',
	'url' => 'https://calicote.dev.iprospective.fr',
	'ws_url' => 'https://calicote.dev.iprospective.fr/modules/mmi_sync/ws.php',
];
protected static $__d_params = [
	'url' => 'https://calicote-erp.dev.iprospective.fr',
	'ws_url' => 'https://calicote-erp.dev.iprospective.fr/custom/mmiprestasync/ws.php',
];

// Mapping
protected static $_maps = [];

protected static $_map_val = [];
protected static $_map_fct = [];

protected static $_p_map_pk = [];
protected static $_d_map_pk = [];

/**
 * Données à utiliser pour le mapping de chaque table.
 * Elle sont utilisées récursivement
 * C'est la méthode de construction/mapping
 */
protected static $_tmap = [
	// Produit simple
	1 => [
		// Tables Doli , données à construire
		'dt' => [
			'name' => 'product',
			// Quelles infos à utiliser ?
			'map_main' => 'product', // c'est la base
			'map_join' => ['product_lang', 'product_shop'], // Children forcément
			// COnstruction des champs enfant
			'children' => [
				// Complément de donnée produit extrafield
				'product_extrafields' => [
					'name' => 'product_extrafields',
					// Rien (de plus) donc identique au parent
				],
				// Association fournisseur/produit avec prix fournisseur
				'product_fournisseur_price' => [
					'name' => 'product_fournisseur_price',
					'map_main' => 'product_supplier',
					'map_model' => 'supplier_price',
					'map_type' => 'list', // mapping list=>list
					'keymap_list' => ['fk_soc', 'ref_fourn'],
					'children' => [
						// Complément de donnée produit extrafield
						'product_fournisseur_price_extrafields' => [
							'name' => 'product_fournisseur_price_extrafields',
							// Rien (de plus) donc identique au parent
						],
					],
				],
			],
		],
		// Tables Presta
		'pt' => [
			// Donnée produit de base
			'name' => 'product',
			// Quelles infos à utiliser ?
			'map_main' => 'product', // c'est la base
			'map_join' => ['product_extrafields'], // Children forcément
			// Construction des champs enfant
			'children' => [
				// Donnée de magasin pour un produit/langue donné
				'product_shop' => [
					'name' => 'product_shop',
				],
				// Donnée de lang/shop pour une produit donné
				'product_lang' => [
					'name' => 'product_lang',
				],
				// Association fournisseur/produit avec prix fournisseur
				'product_supplier' => [
					'name' => 'product_supplier',
					'map_main' => 'product_fournisseur_price',
					'map_model' => 'supplier_price',
					'map_type' => 'list',
					'keymap_list' => ['id_supplier', 'product_supplier_reference'],
				],
			],
		],
	],
	// Produit décliné
	2 => [
		'dt' => [
			'name' => 'product',
			'map_main' => 'product_attribute',
			'map_join' => ['product', 'product_lang', 'product_shop'],
		],
	],
	// Produit composé
	5 => [
		'dt' => [
			'name' => 'product',
			'map_main' => 'product',
			'map_join' => ['product_lang', 'product_shop'],
		],
	],
	// Stock Produit
	12 => [
		'dt' => [
			'name' => 'product_stock',
			'map_main' => 'stock_available',
		],
		'pt' => [
			'name' => 'stock_available',
			'map_main' => 'product_stock',
		],
	],
	// Lot Produit
	7 => [
		'dt' => [
			'name' => 'product_lot',
			'map_main' => 'products_dlc_dluo',
			'children' => [
				// Batch
				'product_batch' => [
					'name' => 'product_batch',
				],
			],
		],
		'pt' => [
			'name' => 'products_dlc_dluo',
			'map_main' => 'product_lot',
			'map_join' => ['product_batch', 'product_stock'],
		],
	],
	// Fournisseur
	3 => [
		'dt' => [
			'name' => 'societe',
			'map_main' => 'supplier',
			'map_join' => ['supplier_lang', 'supplier_shop'],
			'children' => [
				// Adresse client
				'socpeople' => [
					'name' => 'socpeople',
					'map_main' => 'address',
					'keymap_list' => ['fk_soc', 'ref_fourn'],
					'map_model' => 'address',
				],
			],
		],
		'pt' => [
			'name' => 'supplier',
			'map_main' => 'societe',
			'children' => [
				// Donnée de magasin pour un fournisseur donné (simple associatio en fait)
				'supplier_shop' => [
					'name' => 'supplier_shop',
				],
				// Donnée de lang/shop pour un fournisseur donné
				'supplier_lang' => [
					'name' => 'supplier_lang',
				],
				// Adresses
				'address' => [
					'name' => 'address',
					'map_type' => 'list',
					'map_main' => 'socpeople',
					'map_model' => 'address',
				],
			],
		],
	],
	// Client
	4 => [
		'dt' => [
			'name' => 'societe',
			'map_main' => 'customer',
			'children' => [
				// Adresse client
				'socpeople' => [
					'name' => 'socpeople',
					'map_type' => 'list',
					'map_main' => 'address',
					'map_model' => 'address',
				],
			],
		],
		'pt' => [
			'name' => 'customer',
			'map_main' => 'societe',
			'children' => [
				// Adresse client
				'address' => [
					'name' => 'address',
					'map_type' => 'list',
					'map_main' => 'socpeople',
					'map_model' => 'address',
				],
			],
		],
	],
	// Commande / Order
	8 => [
		'dt' => [
			'name' => 'commande',
			'map_main' => 'orders',
			'children' => [
				// Ligne de commande
				'commandedet' => [
					'name' => 'commandedet',
					'map_type' => 'list',
					'map_main' => 'order_detail',
					'map_model' => 'order_detail',
				],
			],
		],
		'pt' => [
			'name' => 'orders',
			'map_main' => 'commande',
			'children' => [
				// Ligne de commande
				'order_detail' => [
					'name' => 'order_detail',
					'map_type' => 'list',
					'map_main' => 'commandedet',
					'map_model' => 'order_detail',
				],
			],
		],
	],
	// Ligne de commande
	9 => [
		'dt' => [
			'name' => 'commandedet',
			'map_main' => 'order_detail',
		],
		'pt' => [
			'name' => 'order_detail',
			'map_main' => 'commandedet',
		],
	],
	// Adresse
	10 => [
		'dt' => [
			'name' => 'socpeople',
			'map_main' => 'address',
		],
		'pt' => [
			'name' => 'address',
			'map_main' => 'socpeople',
		],
	],
	// Prix fournisseur
	11 => [
		'dt' => [
			'name' => 'product_fournisseur_price',
			'map_main' => 'product_supplier',
		],
		'pt' => [
			'name' => 'product_supplier',
			'map_main' => 'product_fournisseur_price',
		],
	],
];

// HELPERS

// @todo mettre ailleurs, c'est un helper bien plus général
public static function _array_merge_recursive(&$map, $tmap)
{
	foreach($tmap as $i=>$j) {
		if (!isset($map[$i]))
			$map[$i] = $j;
		elseif (!is_array($map[$i]) || !is_array($j))
			$map[$i] = $j;
		else
			static::_array_merge_recursive($map[$i], $j);
	}
}

// CLASS

protected static function __init_params()
{
	static::__log(__METHOD__);
	
	static::$_name = substr(get_called_class(), 6);
	$db = static::$__db;
	
	// Model	
	$q = $db->query('SELECT *
		FROM _models
		WHERE ref=\''.static::$_name.'\'');
	//var_dump($q);
	$r = $q->fetch_assoc();
	static::$_id = $r['id'];
	static::$_label = $r['label'];

	
	// Mappings
	
	$sql = 'SELECT *
		FROM `_map`
		WHERE `model_id`='.static::$_id;
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	$maps_ids = [];
	static::$_maps = [];
	while($row=$q->fetch_assoc()) {
		static::$_maps[$row['id']] = array_merge(
			['pt'=>[], 'dt'=>[], 'p_tree'=>[], 'd_tree'=>[]],
			$row
		);
		$maps_ids[] = $row['id'];
	}
	//var_dump(static::$_maps);
	
	// Tables PRESTA
	
	$sql = 'SELECT *
		FROM `_map_p_tables`
		WHERE `map_id` IN ('.implode(', ', $maps_ids).')';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	//var_dump($q); die();
	while($row=$q->fetch_assoc()) {
		//var_dump($row);
		$t = \Table\P_Table::_get($row['tid']);
		if (!isset(static::$_pt[$t->ref])) {
			static::$_pt[$t->ref] = $t;
			static::$_pt_alias[$t->alias] = $t->ref;
		}
		//$row['']
		$row['name'] = $t->ref;
		$row['keymap_list'] = empty($row['keymap_list']) ?[] :explode(',', $row['keymap_list']);
		static::$_maps[$row['map_id']]['pt'][$t->ref] = $row;
	}
	//var_dump(static::$_pt_alias); die();
	//var_dump(static::$_pt);
	
	// Tables DOLI
	
	$sql = 'SELECT *
		FROM `_map_d_tables`
		WHERE `map_id` IN ('.implode(', ', $maps_ids).')';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	//var_dump($q); die();
	while($row=$q->fetch_assoc()) {
		//var_dump($row);
		$t = \Table\D_Table::_get($row['tid']);
		if (!isset(static::$_dt[$t->ref])) {
			static::$_dt[$t->ref] = $t;
			static::$_dt_alias[$t->alias] = $t->ref;
		}
		$row['name'] = $t->ref;
		$row['keymap_list'] = empty($row['keymap_list']) ?[] :explode(',', $row['keymap_list']);
		static::$_maps[$row['map_id']]['dt'][$t->ref] = $row;
	}
	//var_dump(static::$_dt_alias); die();
	//var_dump(static::$_dt); die();
	
	
	// Construction des arbres
	// Obligé de le faire au 2è passage car on utilise des alias dans le mapping fk/pk...
	foreach(static::$_maps as &$map) {
		//echo '<p>Map '.$map['label'].'</p>';
		foreach(['p', 'd'] as $type) {
			// Construction de base
			foreach($map[$type.'t'] as $ref=>&$row) {
				//echo '<p>Table '.$ref.'</p>';
				if (!empty($row['join_params'])) {
					$t = static::${'_'.$type.'t'}[$ref];
					$join_params = explode('=', $row['join_params']);
					// Mappage par défaut
					if(empty($join_params[1])) {
						$join_field = $t->field($join_params[0]);
						//var_dump($row['join_params']); var_dump($join_field);
						$fk_field = $join_field->fk_field;
					}
					// Mappage forcé
					else {
						$fk_field = $join_params[1];
					}
					//var_dump($fk_field);
					list($talias, $tfield) = explode('.', $fk_field);
					$tname = static::${'_'.$type.'t_alias'}[$talias];
					$row['parent'] = $tname;
					$row['join'] = [
						'fieldname'=>$join_params[0],
						'from_tid'=>$t = static::${'_'.$type.'t'}[$tname]->id,
						'from_tname'=>$tname,
						'from_talias'=>$talias,
						'from_fieldname'=>$tfield,
					];
					$map[$type.'t'][$tname]['children'][$ref] = &$row;
				}
			}
		}
	}
	//var_dump(static::$_maps); die();
	
	$tdiff = static::_schema_check(true);
	if (!empty($tdiff)) {
		mail(static::$_notif_email, 'Erreur Schema Synchro', "Les tables suivantes ont un schéma qui diffère :\r\n".implode("\r\n", $tdiff));
		//die();
	}
	//die();
	
	//echo '<p>Mapping complet</p>'; var_dump(static::$_maps); //die();
}

public static function __init()
{
	// Logger
	if (!isset(static::$__logger))
		static::__setlogger($GLOBALS['logger']);
	static::__log(__METHOD__);
	
	// DataBase
	if (!isset(static::$__db))
		static::__setdb($GLOBALS['db']);
	
	// Modèle bien défini !
	if (!isset(static::$_name))
		static::__init_params();
}

public static function __setdb($db)
{
	static::$__db = $db;
}

public static function __db()
{
	return static::$__db;
}

public static function __setlogger($logger)
{
	static::$__logger = $logger;
}

public static function __logger()
{
	return static::$__logger;
}

public static function __log($message, $context=[], $t='notice')
{
	if ($t=='info')
		return static::$__logger->info($message, $context);
	elseif ($t=='warning')
		return static::$__logger->warning($message, $context);
	elseif ($t=='error')
		return static::$__logger->error($message, $context);
	elseif ($t=='critical')
		return static::$__logger->critical($message, $context);
	else
		return static::$__logger->notice($message, $context);
}

public static function _id()
{
	return static::$_id;
}

public static function _name()
{
	return static::$_name;
}

public static function _label()
{
	return static::$_label;
}

// Mappings

public static function _maps()
{
	return static::$_maps;
}

public static function _mapbyobject($type, $otype, $oid=null)
{
	return static::_mapbyotype($type, $otype);
}
public static function _mapbyotype($type, $otype)
{
	//var_dump(static::$_maps); die();
	foreach(static::$_maps as $map_id=>$map) {
		//var_dump($map);
		if ($map[$type.'t_main']==$otype) {
			return $map;
		}
	}
}

// Tables

public static function _pt()
{
	return static::$_pt;
}
public static function _pt_fields_sync()
{
	static::__log(__METHOD__);
	foreach(static::$_pt as $t) {
		var_dump($t);
	}
}

public static function _dt()
{
	return static::$_dt;
}

public static function _schema_check($update=false)
{
	// Check Schema
	$tdiff = [];
	foreach(['p', 'd'] as $type) {
		foreach(static::${'_'.$type.'t'} as $tablename=>$table) {
			$classname = 'Table\\'.$type.'_field';
			$schema_web = static::_ws_action($type, 'schema_fields', ['tablename'=>$tablename]);
			if (!is_null($schema_web)) {
				$schema = $classname::_c_data_get($tablename);
				$diff = static::_data_diff_rows($schema, $schema_web);
				//echo '<p>'.$tablename.'</p>'; var_dump($diff); var_dump($schema_web);
				
				if (!empty($diff)) {
					$tdiff[] = 'Champs sur '.$type.'.'.$tablename;
					//var_dump($diff);
					foreach($diff as $fieldname=>$field) {
						if (!empty($field)) {
							//var_dump($field);
							if ($update) {
								// Insert
								if (empty($field[0]))
									$classname::_c_insert($tablename, $field[1]);
								// Delete
								elseif (empty($field[1]))
									$classname::_c_delete($tablename, $field[0]);
								// Update
								else
									$classname::_c_update($tablename, $field[1]);
							}
						}
					}
				}
			}
			
			$index_web = static::_ws_action($type, 'schema_index', ['tablename'=>$tablename]);
			if (!is_null($index_web)) {
				$index = $classname::_i_data_get($tablename);
				$diff = static::_data_diff_rows($index, $index_web);
				//echo '<p>'.$tablename.'</p>'; var_dump($diff); //var_dump($index_web);
				if (!empty($diff)) {
					$tdiff[] = 'Index sur '.$type.'.'.$tablename;
					//var_dump($diff);
					foreach($diff as $fieldname=>$field) {
						if (!empty($field)) {
							//var_dump($field);
							if ($update) {
								// Insert
								if (empty($field[0]))
									$classname::_i_insert($tablename, $field[1]);
								// Delete
								elseif (empty($field[1]))
									$classname::_i_delete($tablename, $field[0]);
								// Update
								else
									$classname::_i_update($tablename, $field[1]);
							}
						}
					}
				}
			}
		}
	}
	return $tdiff;
}

// Recherche

public static function _select($params=null)
{
	return static::_o_list($params);
}
public static function _o_list($params=null)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	if (!is_array($params))
		$params = [];
	
	$o = [];
	
	// Objects
	$sql = 'SELECT o.*
		FROM _objects AS o
		WHERE o.mid='.static::$_id
		.(isset($params['id']) ?' AND o.id='.$params['id'] :'')
		.(isset($params['ref']) ?' AND o.ref="'.$params['ref'].'"' :'');
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	while ($r = $q->fetch_assoc()) {
		$o[$r['id']] = array_merge(
			['p'=>[], 'd'=>[],],
			$r
		);
	}
	
	// Related Dolibarr tables (with pk!)
	static::_o_list_p($o, $params);
	
	// Related Prestashop tables (with pk!)
	static::_o_list_d($o, $params);
	
	return $o;
}

protected static function _o_list_p(&$o, $params=null)
{
	$db = static::$__db;
	
	foreach(static::$_dt as $t) if($t->f_pk) {
		$sql = 'SELECT ol.*'
			.(!empty($t->f_pk) ?', t.'.$t->f_pk.' AS pk' :'')
			.(!empty($t->f_fk) ?', t.'.$t->f_fk.' AS fk' :'')
			.(!empty($t->f_ref) ?', t.'.$t->f_ref.' AS ref' :'')
			.(!empty($t->f_label) ?', t.'.$t->f_label.' AS label' :'')
			.(!empty($t->f_ts) ?', t.'.$t->f_ts.' AS ts' :'')
			.'
			FROM _objects AS o
			INNER JOIN _objects_d AS ol
				ON ol.id=o.id AND ol.tid='.$t->id.'
			LEFT JOIN .'.DB_D_PREFIX.$t->ref.' AS t
				ON t.'.$t->f_pk.'=ol.oid
			WHERE o.mid='.static::$_id
			.(isset($params['id']) ?' AND o.id='.$params['id'] :'')
			.(isset($params['ref']) ?' AND o.ref="'.$params['ref'].'"' :'');
		if (DEBUG_SQL)
			var_dump($sql);
		$q = $db->query($sql);
		while ($r = $q->fetch_assoc()) {
			//var_dump($r);
			//var_dump($t);
			$o[$r['id']]['dt'][] = $r;
			if (!empty($t->f_pk)) {
				$o[$r['id']]['d']['tid'] = $t->id;
				$o[$r['id']]['d']['oid'] = $r['oid'];
			}
			if (!empty($t->f_ref))
				$o[$r['id']]['d']['ref'] = $r['ref'];
			if (!empty($t->f_label) && !empty($r['label']))
				$o[$r['id']]['d']['label'] = $r['label'];
			if (!empty($t->f_ts))
				$o[$r['id']]['d']['ts'] = $r['ts'];
		}
	}
}

protected function _o_list_d(&$o, $params=null)
{
	$db = static::$__db;
	
	foreach(static::$_pt as $t) if($t->f_pk) {
		//var_dump($j); die();
		$sql = 'SELECT ol.*'
			.(!empty($t->f_pk) ?', t.'.$t->f_pk.' AS pk' :'')
			.(!empty($t->f_fk) ?', t.'.$t->f_fk.' AS fk' :'')
			.(!empty($t->f_ref) ?', t.'.$t->f_ref.' AS ref' :'')
			.(!empty($t->f_label) ?', t.'.$t->f_label.' AS label' :'')
			.(!empty($t->f_ts) ?', t.'.$t->f_ts.' AS ts' :'')
			.(!empty($t->intl) ?', t.id_lang' :'')
			.(!empty($t->shop) ?', t.id_shop' :'')
			.'
			FROM _objects AS o
			INNER JOIN _objects_p AS ol
				ON ol.id=o.id AND ol.tid='.$t->id.'
			LEFT JOIN '.DB_P_PREFIX.$t->ref.' AS t
				ON t.'.$t->f_pk.'=ol.oid
			WHERE o.mid='.static::$_id
			.(isset($params['id']) ?' AND o.id='.$params['id'] :'')
			.(isset($params['ref']) ?' AND o.ref="'.$params['ref'].'"' :'');
		if (DEBUG_SQL)
			var_dump($sql);
		$q = $db->query($sql);
		while ($r = $q->fetch_assoc()) {
			//var_dump($r);
			$o[$r['id']]['pt'][] = $r;
			if (!empty($t->f_pk)) {
				$o[$r['id']]['p']['tid'] = $t->id;
				$o[$r['id']]['p']['oid'] = $r['oid'];
			}
			if (!empty($t->f_ref))
				$o[$r['id']]['p']['ref'] = $r['ref'];
			if (!empty($t->f_label) && !empty($r['label']))
				$o[$r['id']]['p']['label'] = $r['label'];
			if (!empty($t->f_ts))
				$o[$r['id']]['p']['ts'] = $r['ts'];
		}
	}
}

/**
 * Liste des objets Prestashop
 */
public static function _p_list($params=null)
{
	// Overload
}

public static function _p_list_sql($params=null)
{
	return static::_p_sql($params);
}

public static function _p_sql($params=null)
{
	// Overload
}


/**
 * Liste des Objets Dolibarr
 */
public static function _d_list($params=null)
{
	$db = static::$__db;
	
	$sql = static::_d_list_sql($params);
	if (DEBUG_SQL)
		var_dump($sql);
		
	$l = [];
	$q = $db->query($sql);
	while ($r = $q->fetch_assoc()) {
		//var_dump($r);
		$l[] = $r;
	}
	
	return $l;
}

public static function _d_list_sql($params=null)
{
	return static::_d_sql($params);
}

public static function _d_sql($params=null)
{
	// Overload
}

// CRUD

public static function _add($map_id, $ref=NULL, $label=NULL)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	$sql = 'INSERT INTO _objects
		(mid, map_id, ref, label)
		VALUES
		('.static::$_id.', '.$map_id.', '.(is_null($ref) ?'NULL' :'"'.$ref.'"').', '.(is_null($label) ?'NULL' :'"'.$label.'"').')';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	return $db->insert_id;
}
/**
 * Associe une ligne de table à l'objet
 */
public static function _t_assoc($type, $id, $tid, $oid)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	$sql = 'INSERT INTO _objects_'.$type.'
		(id, tid, oid)
		VALUES
		('.$id.', '.$tid.', '.$oid.')';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	
	return $q;
}

/*
 * Get object
 */
public static function _find($id)
{
	return static::_get($id);
}
public static function _get($param)
{
	if (is_numeric($param) && $param==(int)$param)
		return static::_getbyid($param);
	elseif (is_string($param))
		return static::_getbyref($param);
}
public static function _getbyid($id)
{
	static::__log(__METHOD__);
	
	// Récupérer d'abord les objets paramétrés facilement en BDD avec le système PK/FK
	
	// Récupérer ensuite ceux plus complexes
	
	$l = static::_o_list(['id'=>$id]);
	//var_dump($l);
	$o = null;
	if (!empty($l))
		$o = array_pop($l);
	if (DEBUG_SYNCHRO)
		var_dump($o);
	
	return $o;
}
public static function _getbyref($ref)
{
	static::__log(__METHOD__);
	
	$l = static::_o_list(['ref'=>$ref]);
	//var_dump($l);
	$o = null;
	if (!empty($l))
		$o = array_pop($l);
	//var_dump($o);
	
	return $o;
}
public static function _getbyoid($type, $otype, $oid)
{
	$id = static::_getidbyoid($type, $otype, $oid);
	if (DEBUG_SQL)
		var_dump($id);
	if ($id)
		return static::_get($id);
	//var_dump($o); die();
}

/*
 * Get object ID
 */
public static function _getidbyref($ref)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	$sql = 'SELECT o.id
		FROM _objects o
		WHERE o.ref="'.$ref.'"';
	if (DEBUG_SQL)
		var_dump($sql);
	if (!($q = $db->query($sql))) {
		if (DEBUG_SYNCHRO)
			var_dump('COUILLE OID');
		return;
	}
	
	if (list($id)=$q->fetch_row())
		return $id;
}
public static function _getidbyoid($type, $otype, $oid)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	$sql = 'SELECT o.id
		FROM _objects o
		INNER JOIN _objects_'.$type.' ot ON ot.id=o.id
		INNER JOIN _'.$type.'_tables t ON t.id=ot.tid
		WHERE ot.oid='.$oid.'
			AND t.ref="'.$otype.'"';
		// @todo verif que map_id est bien inutile dans tous les cas de figure!
		// AND o.map_id='.$map_id.'
	if (DEBUG_SQL)
		var_dump($sql);
	//trigger_error($sql);
	if (!($q = $db->query($sql))) {
		if (DEBUG_SYNCHRO)
			var_dump('COUILLE OID');
		return;
	}
	
	if (list($id)=$q->fetch_row())
		return $id;
}

/*
 * Check if objects exists
 */
public static function _existsbyoid($type, $otype, $oid)
{
	return is_numeric(static::_getidbyoid($type, $otype, $oid));
}

/**
 * Récupère Un objet
 * ainsi que l'ensemble des données liées/synchronisées des tables locales et distantes
 */
public static function _get_all($id)
{
	$o = static::_get($id);
	//var_dump($o);
	
	// Récupération données Presta
	$o['p_tree'] = [];
	$o['p_data'] = [];
	$o['p_tree_web'] = [];
	$o['p_data_web'] = [];
	static::_get_all_p($o);
	
	// Récupération données Doli
	$o['d_tree'] = [];
	$o['d_data'] = [];
	$o['d_tree_web'] = [];
	$o['d_data_web'] = [];
	static::_get_all_d($o);
	
	// Transformation arbre => linéarisation en tables/rows
	//static::_tree2data($o);
	
	//var_dump($o);
	return $o;
}
public static function _get_all_p(&$o)
{
	$type = 'p';
	if (!isset($o[$type]['oid']))
		return;
	
	// Table Mapping
	$map_id = $o['map_id'];
	$map = static::$_maps[$map_id];
	// Main table
	$t_main = $map[$type.'t_main'];
	
	$oid = $o[$type]['oid'];
	
	$o[$type.'_tree'] = static::_get_data_tree($map_id, $type, $t_main, $oid);
	$o[$type.'_tree_web'] = static::_ws_model($type, $map_id, $oid);
	//var_dump($o[$type.'_tree_web']); die();
}
public static function _get_all_d(&$o)
{
	$type = 'd';
	if (!isset($o[$type]['oid']))
		return;
	
	// Table Mapping
	$map_id = $o['map_id'];
	$map = static::$_maps[$map_id];
	// Main table
	$t_main = $map[$type.'t_main'];
	
	$oid = $o[$type]['oid'];
	
	$o[$type.'_tree'] = static::_get_data_tree($map_id, $type, $t_main, $oid);
	$o[$type.'_tree_web'] = static::_ws_model($type, $map_id, $oid);
	//var_dump($o[$type.'_tree_web']); die();
}

// Construction Arbre de données

/**
 * ATTENTION! : params doit être construit en amont
 */
public static function _get_data_tree($map_id, $type, $tablename, $params)
{
	$maps = static::$_maps[$map_id][$type.'t'];
	$map = $maps[$tablename];
	$table = static::${'_'.$type.'t'}[$tablename];
	//var_dump($table); die();
	//var_dump($map); die();
	
	// Récupération données de base
	$datainit = [
		'tablename' => $tablename,
		'params' => $params,
		'join_type' => $map['join_type'],
		'children' => [],
	];
	
	if ($map['join_type'] == 'row') {
		//echo '<p>'.$tablename.'</p>';
		//var_dump(array_keys(static::${'_'.$type.'t'}));
		if ($row = $table->get($params)) {
			return static::_get_data_tree_sub($type, $map, $datainit, $row);
		}
	}
	elseif ($map['join_type']  == 'list') {
		$rows = $table->select($params);
		//var_dump($rows); die();
		$datas = [];
		foreach($rows as $key=>$row) {
			$key_mapped = static::_map_key($type, $map, $row, $key);
			if (DEBUG_SYNCHRO) {
				var_dump($key_mapped);
				var_dump($row);
			}
			$datas[$key_mapped] = static::_get_data_tree_sub($type, $map, $datainit, $row);
		}
		return $datas;
	}
}
/**
 * Sous-requête
 * Juste histoire de factoriser et simplifier le code
 */
public static function _get_data_tree_sub($type, $map, $data, $row)
{
	// Affectation row au dataset
	$data['row'] = $row;
	
	// Mise à jour param
	$tablename = $data['tablename'];
	$table = static::${'_'.$type.'t'}[$tablename];
	if (!empty($pk_fields = $table->pk_fields())) {
		if (DEBUG_SYNCHRO) {
			echo '<p>_get_data_tree_sub : '.$tablename.'</p>';
			var_dump($pk_fields);
			var_dump($data['params']);
		}
		foreach($pk_fields as $fieldname) {
			if (!empty($row[$fieldname])) {
				//var_dump($fieldname);
				//var_dump($row[$fieldname]);
				if (is_array($data['params']))
					$data['params'][$fieldname] = $row[$fieldname];
			}
		}
	}
	//var_dump($data); die();
	
	// récursion enfants
	if (!is_null($row) && !empty($map['children'])) foreach($map['children'] as $tname=>$t) {
		//$table = static::${'_'.$type.'t'}[$tname];
		//echo '<p>'.$tname.'</p>'; var_dump($t);// die();
		$params = [
			$t['join']['fieldname'] => $row[$t['join']['from_fieldname']],
		];
		if (!empty($t['params'])) {
			$tparams = explode(',', $t['params']);
			foreach($tparams as $i)
				$params[$i] = static::${'__'.$type.'_params'}[$i];
		}
		$data['children'][$tname] = static::_get_data_tree($map['map_id'], $type, $tname, $params);
	}
	
	return $data;
}

// WEBSERVICE

/**
 * Parse data model from webservice to makle it similmar to 
 */
public static function _parse_model($type, $map_id, &$o)
{
	$tablename = $o['tablename'];
	$map = static::$_maps[$map_id][$type.'t'][$tablename];
	if (DEBUG_SYNCHRO)
		var_dump($map);
	if (!empty($o['children'])) foreach($o['children'] as $tname=>&$rows) {
		$tmap = static::$_maps[$map_id][$type.'t'][$tname];
		if (DEBUG_SYNCHRO)
			var_dump($tmap);
		if ($tmap['join_type']=='list' && !empty($tmap['map_model'])) foreach($rows as $key=>&$row) {
			$key_mapped = static::_map_key($type, $tmap, $row, $key);
			if (DEBUG_SYNCHRO)
				var_dump($key_mapped);
			if ($key_mapped != $key) {
				unset($rows[$key]);
				$rows[$key_mapped] = $row;
			}
		}
	}
	
	if (DEBUG_SYNCHRO)
		var_dump($o);
	//die();
}

/**
 * Webservice Specific actions
 */
public static function _ws_model($type, $map_id, $id)
{
	//var_dump(static::$_maps[$map_id]); die();
	$o = static::_ws_action($type, 'get', ['id'=>$id, 'map'=>static::$_maps[$map_id]]);
	static::_parse_model($type, $map_id, $o);
	return $o;
}
public static function _ws_odata($type, $tablename, $oid)
{
	$r = static::_ws_action($type, 'getrow', ['tablename'=>$tablename, 'id'=>$oid]);
	var_dump($r);
	die();
	//var_dump(static::$_maps[$map_id]);
	return static::_ws_action($type, 'getrow', ['tablename'=>$tablename, 'id'=>$oid]);
}
public static function _ws_oexists($type, $tablename, $oid)
{
	//var_dump(static::$_maps[$map_id]);
	return static::_ws_action($type, 'exists', ['tablename'=>$tablename, 'id'=>$oid]);
}
public static function _ws_oexists_ref($type, $tablename, $ref)
{
	//var_dump(static::$_maps[$map_id]);
	return static::_ws_action($type, 'exists_ref', ['tablename'=>$tablename, 'ref'=>$ref]);
}
public static function _ws_sql($type, $sql_type, $sql)
{
	return static::_ws_action($type, 'sql', ['type'=>$sql_type, 'sql'=>$sql]);
}
/**
 * Webservice action
 */
public static function _ws_action($type, $action, $params=null)
{
	return static::_ws_url($type, '?action='.$action, $params);
}
/**
 * Webservice URL => JSON
 */
public static function _ws_url($type, $url, $post=null)
{
	$url = static::${'__'.$type.'_params'}['ws_url'].$url;
	
	$timeout = 30;
	$useragent = "DoliPrestaSync API";
	
	$ch = curl_init($url); // initialize curl with given url
	//curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER["HTTP_USER_AGENT"]); // set  useragent
	curl_setopt($ch, CURLOPT_USERAGENT, $useragent);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); // write the response to a variable
	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false); // follow redirects if any
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout); // max. seconds to execute
	curl_setopt($ch, CURLOPT_FAILONERROR, 1); // stop when it encounters an error
	if (!empty($post)) {
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post));
	}
	$raw_data = @curl_exec($ch);
	$data = json_decode($raw_data, true);
	if (DEBUG_SYNCHRO || DEBUG_WS) {
		static::__log([$type, $url, $post, $raw_data, $data], [__FILE__, __METHOD__, __LINE__]);
		var_dump($type);
		var_dump($url);
		var_dump($post);
		var_dump($raw_data);
		var_dump($data);
	}
	return $data;
}

// DIFF

public static function _data_diff_rows($rows, $rows_web)
{
	$list = [];
	
	if (isset($rows_web)) foreach($rows_web as $id=>$row) {
		// Update
		if (isset($rows[$id])) {
			$diff = static::_data_diff_row($rows[$id], $row);
			if (!empty($diff))
				$list[$id] = [0=>$rows[$id], 1=>$row, 'diff'=>$diff];
		}
		// Insert
		else {
			$list[$id] = [1=>$row];
		}
	}
	if (isset($rows)) foreach($rows as $id=>$row) {
		// Delete
		if (!isset($rows_web[$id])) {
			$list[$id] = [0=>$row];
		}
	}
	
	return $list;
}
public static function _data_diff_row($row, $row_web)
{
	if (empty($row_web))
		return [];
	
	$diff = [];
	
	foreach($row_web as $name=>$value) {
		//die($name);
		// @todo ne pas tenir compte des dates de mise à jour ..?
		if ($value !== $row[$name]) {
			$diff[$name] = $value;
		}
	}
	
	return $diff;
}


// UPDATE effectif SQL/Webservice

/**
 * Requête SQL depuis Tree sur un set de données avec un mapping donné
 * @todo Fonctionne bien pour une synchronisation, moins bien pour un mapping, mais on va s'arranger!
 * 
 * @param $map Mapping concerné
 * @param $type p/d
 * @param $tree Data locale
 * @param $tree_web Data prod distante
 * @param $tree_new Data nouvelle (à mettre à jour vers distant)
 */
public static function _sql_update($map, $type, $tree, $tree_web, $tree_new=null)
{
	$db = static::$__db;
	//trigger_error('mm');
	
	// On synchronise le distant par màj/mapping
	if (isset($tree_new)) {
		$tree_orig = $tree_new;
		$tree_dest = $tree_web;
		$dest = 'ws';
	}
	// On synchronise le local avec le distant
	else {
		$tree_orig = $tree_web;
		$tree_dest = $tree;
		$dest = 'local';
	}
	
	$tablename = $map['name'];
	if (DEBUG_SYNCHRO || DEBUG_SQL) {
		echo '<p>_sql_update : '.$tablename.'</p>';
		var_dump($map);
		var_dump($tree_orig);
		var_dump($tree_dest);
		var_dump($dest);
		//die();
	}
	$table = static::${"_".$type."t"}[$tablename];
	
	// Rien sur le distant
	if (empty($tree_dest['row'])) {
		// Création depuis distant
		if (!empty($tree_orig['row'])) {
			$sql = $table->data_insert_sql($tree_orig['row'], $tree_orig['params']);
			if (DEBUG_SQL)
				var_dump($sql);
			if ($dest=='local') {
				$q = $db->query($sql);
				//var_dump($q);
				$id = $db->insert_id;
				$sql_notif = 'INSERT INTO _'.$type.'_log (ts, tid, oid, rev, action, fields, details) VALUES (NOW(), '.$map['tid'].', '.$id.', 0, "c", "'.addslashes(json_encode(array_keys($tree_orig['row']))).'", "'.addslashes(json_encode($tree_orig['row'])).'")';
				if (DEBUG_SQL)
					var_dump($sql_notif);
				$q = $db->query($sql_notif);
				$r = ['response'=>!empty($id), 'id'=>$id];
				if (DEBUG_SYNCHRO || DEBUG_SQL)
					var_dump($r);
			}
			elseif ($dest=='ws') {
				$r = static::_ws_sql($type, 'c', $sql);
				if (DEBUG_SYNCHRO || DEBUG_SQL)
					var_dump($r);
			}
		}
	}
	// Rien sur le distant
	elseif (empty($tree_orig['row']) && empty($tree_orig['params'])) {
		// Suppression du local
		if (!empty($tree_dest['row'])) {
			$sql = $table->data_delete_sql($tree_dest['params']);
			if (DEBUG_SQL)
				var_dump($sql);
			if ($dest=='local') {
				$q = $db->query($sql);
				$affected_rows = $db->affected_rows;
				$sql_notif = 'INSERT INTO _'.$type.'_log (ts, tid, oid, rev, action) VALUES (NOW(), '.$map['tid'].', '.$tree_dest['params'].', 0, "d")';
				if (DEBUG_SQL)
					var_dump($sql_notif);
				$q = $db->query($sql_notif);
				$r = ['response'=>$affected_rows>0];
				if (DEBUG_SYNCHRO || DEBUG_SQL)
					var_dump($r);
			}
			elseif ($dest=='ws') {
				$r = static::_ws_sql($type, 'd', $sql);
				if (DEBUG_SYNCHRO || DEBUG_SQL)
					var_dump($r);
			}
		}
	}
	// Update 1 => 0 (distant => local)
	else {
		$diff = static::_data_diff_row($tree_dest['row'], $tree_orig['row']);
		//var_dump($diff);
		if (!empty($diff)) {
			// On a bien les params de tree_dest car on va modifier tree_dest !
			$sql = $table->data_update_sql($diff, $tree_dest['params']);
			if (DEBUG_SQL)
				var_dump($sql);
			if ($dest=='local') {
				$q = $db->query($sql);
				$affected_rows = $db->affected_rows;
				if (is_numeric($tree_orig['params'])) {
					$sql_notif = 'INSERT INTO _'.$type.'_log (ts, tid, oid, rev, action, fields, details) VALUES (NOW(), '.$map['tid'].', '.$tree_orig['params'].', 0, "u", "'.addslashes(json_encode(array_keys($diff))).'", "'.addslashes(json_encode($diff)).'")';
					$q = $db->query($sql_notif);
					if (DEBUG_SQL)
						var_dump($sql_notif);
				}
				$r = ['response'=>$affected_rows>0];
			}
			elseif ($dest=='ws') {
				$r = static::_ws_sql($type, 'u', $sql);
				if (DEBUG_SYNCHRO || DEBUG_SQL)
					var_dump($r);
			}
		}
		else {
			$r = ['response'=>false, 'reason'=>'diff_none'];
		}
	}
	
	// On va potentiellement utiliser par la suite des id sur les objets nouvellement créés !
	
	//var_dump($map['children']); die();
	if (isset($map['children'])) foreach($map['children'] as $tablename=>$tmap) {
		if (DEBUG_SYNCHRO) {
			echo '<p>sql_update child '.$tablename.'</p>';
			//var_dump($tree['children'][$tablename]);
			var_dump($tmap);
		}
		// Row simple
		if ($tmap['join_type']=='row') {
			static::_sql_update(
				$tmap,
				$type,
				isset($tree['children'][$tablename]) ?$tree['children'][$tablename] :null,
				isset($tree_web['children'][$tablename]) ?$tree_web['children'][$tablename] :null,
				isset($tree_new['children'][$tablename]) ?$tree_new['children'][$tablename] :null
			);
		}
		// Liste
		else {//if ($tmap['join_type']=='list') {
			// @todo key mapping des listes... chaud patate!
			// Update mapping => serveur
			if (DEBUG_SYNCHRO) {
				if (isset($tree['children'][$tablename]))
					var_dump($tree['children'][$tablename]);
				if (isset($tree_web['children'][$tablename]))
					var_dump($tree_web['children'][$tablename]);
				if (isset($tree_new['children'][$tablename]))
					var_dump($tree_new['children'][$tablename]);
			}
			if (!empty($tree_new['children'][$tablename])) {
				//die();
				foreach($tree_new['children'][$tablename] as $key=>$row) {
					// @todo key mapping
					static::_sql_update(
						$tmap,
						$type,
						isset($tree['children'][$tablename][$key]) ?$tree['children'][$tablename][$key] :null,
						isset($tree_web['children'][$tablename][$key]) ?$tree_web['children'][$tablename][$key] :null,
						$row
					);
				}
			}
			// Mise à jour serveur => local
			else {
				if (DEBUG_SYNCHRO) {
					var_dump($tree_web['children'][$tablename]);
				}
				if (!empty($tree_web['children'][$tablename])) {
					foreach($tree_web['children'][$tablename] as $key=>$row) {
						static::_sql_update(
							$tmap,
							$type,
							isset($tree['children'][$tablename][$key]) ?$tree['children'][$tablename][$key] :null,
							$row,
							null
						);
					}
				}
			}
		}
	}
	
	return isset($r) ?$r :null;
}


// UPDATE

// Mise à jour à partir d'une DIFF
// @Todo Attention de bien spécifier le sens de synchro ! local/distant !!
/*
 * Synchro P=>P
 * P web => P local
 */
public static function _p_update(&$o)
{
	$map_id = $o['map_id'];
	$map = static::$_maps[$map_id];
	$type = 'p';
	//var_dump($map); die();
	// @todo : checker les éventuels désynchro avant de balancer la sauce...
	// Appliquer les modifs uniquement lorsque l'autre partie est synchro en webservice.
	$r = static::_sql_update($map[$type.'t'][$map[$type.'t_main']], $type, $o[$type.'_tree'], $o[$type.'_tree_web']);
	if (DEBUG_SQL || DEBUG_SYNCHRO) {
		var_dump($r);
	}
	
	return $r;
}
/*
 * Synchro D=>D
 * D web => D local
 */
public static function _d_update(&$o)
{
	$map_id = $o['map_id'];
	$map = static::$_maps[$map_id];
	$type = 'd';
	//var_dump($o[$type.'_tree_web']); die();
	// @todo : checker les éventuels désynchro avant de balancer la sauce...
	// Appliquer les modifs uniquement lorsque l'autre partie est synchro en webservice.
	$r = static::_sql_update($map[$type.'t'][$map[$type.'t_main']], $type, $o[$type.'_tree'], $o[$type.'_tree_web']);
	if (DEBUG_SQL || DEBUG_SYNCHRO) {
		var_dump($r);
	}
	
	return $r;
}
/*
 * Mappage P=>D
 * P web => D Web
 */
public static function _pd_update(&$o, &$update) //, $diff
{
	// @todo : checker les éventuels désynchro avant de balancer la sauce...
	// Appliquer les modifs uniquement lorsque l'autre partie est synchro en webservice.
	$map_id = $o['map_id'];
	$map = static::$_maps[$map_id];
	$type = 'd';
	
	$r = static::_sql_update($map[$type.'t'][$map[$type.'t_main']], $type, $o[$type.'_tree'], $o[$type.'_tree_web'], $update['tree']);
	if (DEBUG_SQL || DEBUG_SYNCHRO) {
		var_dump($r);
	}
	
	return $r;
}
/*
 * Mappage D=P
 * D web => P Web
 */
public static function _dp_update(&$o, &$update) //, $diff
{
	// @todo : checker les éventuels désynchro avant de balancer la sauce...
	// Appliquer les modifs uniquement lorsque l'autre partie est synchro en webservice.
	$map_id = $o['map_id'];
	$map = static::$_maps[$map_id];
	$type = 'p';
	
	$r = static::_sql_update($map[$type.'t'][$map[$type.'t_main']], $type, $o[$type.'_tree'], $o[$type.'_tree_web'], $update['tree']);
	if (DEBUG_SYNCHRO || DEBUG_SQL) {
		var_dump($r);
	}
	
	return $r;
	
	$sqls = static::_diff_sql($o['map_id'], 'p', $diff);
	if (DEBUG_SYNCHRO || DEBUG_SQL)
		var_dump($sqls);
	return $sqls;
}


// Helpers

/**
 * Text to slug
 * @return string
 */
public static function _slugify($text, string $divider='-')
{
	// Remove HTML entities
	$text = html_entity_decode($text);

	// replace non letter or digits by divider
	$text = preg_replace('~[^\pL\d]+~u', $divider, $text);

	// transliterate
	$text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);

	// remove unwanted characters
	$text = preg_replace('~[^-\w]+~', '', $text);

	// trim
	$text = trim($text, $divider);

	// remove duplicate divider
	$text = preg_replace('~-+~', $divider, $text);

	// lowercase
	$text = strtolower($text);

	if (empty($text)) {
		return 'n-a-'.rand(1, 1000000000);
	}

	return $text;
}


// SYNCHRO

/**
 * Synchro Presta <=> Doli
 */
public static function _sync($sens, $id)
{
	static::__log(__METHOD__);
	$source = ($sens=='dp') ?'d' :'p';
	$dest = ($sens=='dp') ?'p' :'d';
	
	// On récupère le dataset complet de chaque côté
	$o = static::_get_all($id);
	//var_dump($o); die();
	//die();
	
	// Vérification que la destination est synchrone
	$fct = '_'.$dest.'_diff';
	//static::$fct($o);
	// if != null => die
	
	// Mise à jour locale, après distant s'il s'est bien exécuté
	// SQL direct OK ici car on réplique des infos
	$fct = '_'.$source.'_update';
	static::$fct($o);
	
	// Mapping vers distant
	$update = static::_map($o, $sens);
	if (DEBUG_SYNCHRO) {
		echo '<p>UPDATE MAPPING :</p>';
		var_dump($update);
		var_dump($o);
		//die();
	}
	
	// Mise à jour dest distante
	$fct = '_'.$sens.'_update';
	$r = static::$fct($o, $update);
	// Création objet => Association
	if (!empty($r['id']))
		static::_t_assoc($dest, $id, $update['tid'], $r['id']);
	
	if (DEBUG_SYNCHRO || DEBUG_SQL)
		var_dump($r);
	
	// Synchronisation destination après mise à jour
	$fct = '_'.$dest.'_update';
	static::$fct($o);
	
	// Mise à jour dest locale depuis distant
}


// MAPPING

/**
 * Mapping de valeurs statiques
 * To overload
 */
public static function _map_val_set($type)
{
	$timestamp = time();
	$datetime = date('Y-m-d H:i:s', $timestamp);
	
	static::$_map_val = [
		'd' => [
			'timestamp' => $timestamp,
			'datetime' => $datetime,
			'lang' => 'fr',
			// Dolibarr
			'id_user_author' => 1, // admin ?
			'id_entrepot' => 1, // Entrepot par défaut
			'id_currency' => 1, // Monnaie
			//'tva_tx' => 20, // TVA
		],
		'p' => [
			'timestamp' => $timestamp,
			'datetime' => $datetime,
			'lang' => 'fr',
			// Prestashop
			'id_lang' => P_LANG_ID,
			'id_shop' => P_SHOP_ID,
		],
	];
}

public static function _map_fk($type, $modelname, $tablename, $oid)
{
	$orig = $type;
	$dest = $orig=='p' ?'d' :'p';
	require_once MODEL_PATH.'/'.$modelname.'.php';
	$classname = '\\Model\\'.$modelname;
	$o = $classname::_getbyoid($orig, $tablename, $oid);
	if (DEBUG_SYNCHRO) {
		var_dump("$classname::_getbyoid('$orig', $tablename, $oid)");
		var_dump($o);
		var_dump($o[$dest]);
	}
	return $o;
}

/**
 * Mapping de clé de liste d'objet
 * Selon les cas :
 * - ~int => clé primaire pk de table
 * - #int => clé d'objet mappé
 * - -blablabla => issu d'un mapping
 * Le reste on n'en tiens pas compte
 */
public static function _map_key($type, &$map, &$row, $key=null)
{
	$tablename = $map['name'];	
	//trigger_error('ff');
	
	// Modèle => On teste si lié
	if (!empty($map['map_model'])) {
		$model = $map['map_model'];
		$modelclass = '\\Model\\'.$model;
		require_once MODEL_PATH.'/'.$model.'.php';
		$id = $modelclass::_getidbyoid($type, $tablename, $key);
		if (DEBUG_SYNCHRO)
			var_dump($id);
		if (is_numeric($id))
			$key_mapped = '#'.$id;
		else
			$key_mapped = $key;
	}
	// Mapping à la mano à l'aide de plusieurs champs
	elseif (!empty($map['keymap_list'])) {
		$key_mapped_list = [];
		foreach($map['keymap_list'] as $i)
			$key_mapped_list[] = $row[$i];
		
		$key_mapped = '-'.implode('-', $key_mapped_list);
	}
	// Natif
	else {
		$key_mapped = $key;
	}
	
	if (DEBUG_SYNCHRO) {
		var_dump($map);
		var_dump($tablename);
		var_dump($row);
		var_dump($key);
		var_dump($key_mapped);
		//die();
	}
	return $key_mapped;
}

/**
 * Mapping Presta <=> Doli
 * à partir d'un set de différences
 * On génère un set de différences
 */
public static function _map(&$o, $sens)
{
	$orig = $sens=='pd' ?'p' :'d';
	$dest = $type = $sens=='pd' ?'d' :'p';
	//$_t = ($sens == 'pd') ?static::$_dt :static::$_pt;
	//$_t_alias = ($sens == 'pd') ?static::$_dt_alias :static::$_pt_alias;
	
	//var_dump($o); die();
	// Liste tables associées au modèle
	//var_dump(static::$_pt); var_dump(static::$_dt);
	
	// Données pratiques par défaut
	static::_map_val_set($type);
	
	// @todo récupérer le bon mapping, qui est actuellement en
	// Dans d_map_fields
	
	$map_id = $o['map_id'];
	$map = static::$_maps[$map_id];
	//echo '<p>Mapping :</p>'; var_dump($map); die();
	
	// Je dois construire un objet en ajoutant des choses à ce que j'ai dans la destination (voir tout)
	// ou bien je fais un diff directement ..?
	
	// Tree 2 Data de base
	// Pour chaque ligne, il faut un contexte différent !
	// product => need product, product_lang, product_shop
	// product_extrafield => need product, product_lang, product_shop
	// product_supplier => need product, supplier
	// => Il faut un autre "mapping" des tables concernées par un enregistrement, et comment cela s'articule
	
	// On commence par la table maitre
	$tablename = $map[$dest.'t_main'];
	var_dump(static::${'_'.$dest.'t'});
	$table = static::${'_'.$dest.'t'}[$tablename];
	//var_dump($table); die();
	
	$data = [
		'model_id' => static::$_id,
		'model_name' => static::$_name,
		'map_id' => $map_id,
		'id' => $o['id'],
		'p' => $o['p'],
		'd' => $o['d'],
		//'o' => $o,
		't_main' => $tablename,
		'tid' => $table->id,
		'tree'=>[],
	];
	//return $data;
	
	// Bouclage sur les tables de la destination présentes dans le modèle
	// A priori tout peut être mis à jour, ou créé !
	// on va comparer avec ce qui a été modifié et qui intervient
	
	// @todo : Bon en fait il faut créer plusieurs séries de modifications, ou du moins les hiérarchiser/ordonner
	// Certaines tables nécessitent que d'autre soient créées...
	
	// Data source to use for mapping
	$map_dest = $map[$dest.'t'][$tablename];
	if (isset(static::$_tmap[$map_id][$dest.'t']))
		static::_array_merge_recursive($map_dest, static::$_tmap[$map_id][$dest.'t']);
	//var_dump($map_dest); die()
	
	$tree_dest = $o[$dest.'_tree_web'];
	//var_dump($tree_dest); die();
	
	$tree_orig = $o[$orig.'_tree_web'];
	//var_dump($tree_orig); die();
	
	// initialisation
	$data_from = [];
	
	// @todo : générer $params en amont
	//var_dump($o[$dest]); die();
	if (isset($o[$dest]['oid'])) {
		$params = [
			$table->f_pk => $o[$dest]['oid'],
		];
	}
	else {
		$params = NULL;
	}
	//var_dump($params);
	
	$data['tree'] = static::_map_data_tree($sens, $map_dest, $tree_dest, $tree_orig, $data_from, $params);
	
	return $data;
}

/**
 * Construction d'un arbre récursif de mapping
 * @param $o objet concerné
 * @param $sens sens du mapping
 * @param $map mapping (en fait, en arbre, où on en est du mapping)
 * @param $tree_dest arbre d'objet final actuel, à titre de comparaison
 * @param $tree_dest arbre d'objet d'origine actuel, permettant de construire les données
 * @param $data_from les données sources, que l'on va potentiellement augmenter récursivement avec $tree_dest
 * @param $params les paramètres pk/uk du/des row(s)
 */
public static function _map_data_tree($sens, $map_dest, $tree_dest, $tree_orig, $data_from=[], $params=[])
{
	$dest = $type = $sens=='pd' ?'d' :'p';
	$orig = $sens=='pd' ?'p' :'d';
	
	$tablename = $map_dest['name'];
	
	//var_dump($tablename); var_dump($tree_orig); die();
	
	// Construction/Récupération données de base
	$datainit = [
		'tablename' => $tablename,
		'params' => $params,
		'join_type' => $map_dest['join_type'],
		'children' => [],
	];
	
	if (DEBUG_SYNCHRO) {
		echo '<p>_map_data_tree : '.$tablename.' : map_dest :</p>';
		var_dump($map_dest);
		//die();
	}
	
	// Une ligne de donnée
	if ($map_dest['join_type']=='row') {
		// construction/augmentation $data_from
		if (!empty($map_dest['map_main'])) {
			if ($tree_orig['tablename']==$map_dest['map_main'])
				$data_from[$map_dest['map_main']] = $tree_orig['row'];
		}
		if (!empty($map_dest['map_join'])) foreach($map_dest['map_join'] as $tablename_new) {
			if (isset($tree_orig['children'][$tablename_new]))
				$data_from[$tablename_new] = $tree_orig['children'][$tablename_new]['row'];
		}
		if (DEBUG_SYNCHRO) {
			echo '<p>ROW:Données sources (data_from) '.$tablename.' :</p>';
			//trigger_error('grrr');
			var_dump($data_from);
			var_dump($tree_orig);
			//die();
		}
		
		//echo '<p>'.$tablename.'</p>';
		//var_dump(array_keys(static::${'_'.$type.'t'}));
		//$row = $table->get($params);
		$row = static::_map_table_row($sens, $tablename, $map_dest, $tree_dest, $data_from);
		//var_dump($row); //die();
		return static::_map_data_tree_sub($sens, $map_dest, $tree_dest, $tree_orig, $data_from, $datainit, $row);
	}
	// Liste de données
	else { //if ($map_dest['join_type']=='list') {
		$datas = [];
		// construction/augmentation $data_from
		foreach($tree_orig as $key=>$tree_data) {
			$data_from_sub = array_merge($data_from, [$map_dest['map_main'] => $tree_data['row']]);
			// Sous-tables de la table de liaison
			// @todo
			if (false && !empty($map_dest['map_join'])) foreach($map_dest['map_join'] as $tablename_new) {
				if (isset($tree_orig['children'][$tablename_new]))
					$data_from_sub[$tablename_new] = $tree_orig['children'][$tablename_new]['row'];
			}
			
			$row = static::_map_table_row($sens, $tablename, $map_dest, isset($tree_dest[$key]) ?$tree_dest[$key] :null, $data_from_sub);
			if (DEBUG_SYNCHRO) {
				echo '<p>LIST:Données sources (data_from) '.$tablename.' :</p>';
				var_dump($key);
				var_dump($data_from);
				var_dump($data_from_sub);
				var_dump($tree_data);
				var_dump($row);
				//die();
			}
			
			$key_mapped = static::_map_key($dest, $map_dest, $row, $key);
			//var_dump($key_mapped); die();
			
			$tree = static::_map_data_tree_sub($sens, $map_dest, $tree_dest, $tree_data, $data_from_sub, $datainit, $row);
			//var_dump($tree);
			$datas[$key_mapped] = $tree;
		}
		return $datas;
	}
}
/**
 * Sous-requête
 * Juste histoire de factoriser et simplifier le code
 * 
 * @param $sens
 * @param $map_dest mapping de l'objet à construire
 * @param $tree_dest arbre de donnée final/destination à titre de repère/comparaison
 * @param $tree_orig arbre de donnée source/origine
 * @param $data_from tableau de données source/origine
 * @param $data donnée qu'on va modifier/construire et renvoyer
 * @param $row ligne de donnée de l'objet en cours
 */
public static function _map_data_tree_sub($sens, $map_dest, $tree_dest, $tree_orig, $data_from, &$data, $row)
{
	//$dest = $type = $sens=='pd' ?'d' :'p';
	//$orig = $sens=='pd' ?'p' :'d';
	
	$data['row'] = $row;
	// récursion enfants
	if (!is_null($row) && !empty($map_dest['children'])) foreach($map_dest['children'] as $tablename=>$map_dest_child) {
		$vname = $map_dest_child['join']['from_fieldname'];
		
		$tree_dest_child = isset($tree_dest['children'][$tablename]) ?$tree_dest['children'][$tablename] :[];
		
		// Passage/Mapping de paramètre pour la mise à jour
		// Identique à l'objet existant
		if (isset($tree_dest_child['params'])) {
			$params = $tree_dest_child['params'];
		}
		// Construit nouveau
		elseif (isset($row[$vname])) {
			$params = [
				$map_dest_child['join']['fieldname'] => $row[$vname],
			];
		}
		// Existant
		elseif (isset($tree_dest['row'][$vname])) {
			$params = [
				$map_dest_child['join']['fieldname'] => $tree_dest['row'][$vname],
			];
		}
		// A construire plus tard
		else {
			//$params = [
			//	$map_dest_child['join']['fieldname'] => $vname,
			//];
			$params = [];
		}
		
		if (DEBUG_SYNCHRO) {
			var_dump($map_dest);
			echo '<p>'.$tablename.'</p>';
			var_dump($map_dest_child);
			var_dump($map_dest_child['join']['fieldname']);
			var_dump($vname);
			echo '<p>tree dest '.$tablename.'</p>';
			var_dump($tree_dest_child);
			var_dump($params);
		}
		
		// @todo y'a un truc à faire, je fais deux fois le test y'a moyen de factoriser
		
		//var_dump($map_dest_child);
		if ($map_dest_child['join_type']=='row') {
			$tree_orig_child = $tree_orig;
		}
		else {//if ($map_dest_child['join_type']=='list') {
			if (DEBUG_SYNCHRO) {
				var_dump(isset($map_dest_child['map_main']));
				//die();
			}
			if (!isset($map_dest_child['map_main']))
				continue;
			//var_dump($tree_orig['children']);
			$tree_orig_child = isset($tree_orig['children'][$map_dest_child['map_main']]) ?$tree_orig['children'][$map_dest_child['map_main']] :[];
		}
		
		$data['children'][$tablename] = static::_map_data_tree($sens, $map_dest_child, $tree_dest_child, $tree_orig_child, $data_from, $params);
	}
	
	return $data;
}

/**
 * Mapping Presta <=> Doli
 * Concernant une donnée (ligne) d'une table
 * Il s'agit du cas de base
 * On aura plusieurs types de mapping :
 * - valeur par défaut
 * - récupération/modification d'une valeur
 * - récupération/mappage d'une clé à partir de la table de mappage de clés
 *
 * @param $sens
 * @param $tablename
 * @param $map_dest Mapping que l'on construit
 * @param $tree_dest Arbre de donnée actuel sur lequel on peut se baser/comparer
 * @param $data_from Données source permettant de construre la donnée à retourner
 */
public static function _map_table_row($sens, $tablename, $map_dest, $tree_dest, $data_from)
{
	$type = $dest = ($sens=='pd') ?'d' :'p';
	$orig = ($sens=='pd') ?'p' :'d';
	$table = static::${'_'.$dest.'t'}[$tablename];
	//var_dump($tree_dest); var_dump($map_dest); die();
	$u = [];
	if (DEBUG_SYNCHRO) {
		echo '<p>map table row :</p>';
		var_dump($tablename);
		var_dump($map_dest);
		var_dump($tree_dest);
		var_dump($data_from);
		//var_dump($table);
	}
	//var_dump($p_diff); die();
	//var_dump($p_data[$tablename]);
	//var_dump($d_data); die();
	
	// Bouclage sur les champs présent dans la tables du modèle
	foreach($table->map_fields($map_dest['map_id']) as $fieldname=>$field) {
		
		if (DEBUG_SYNCHRO) {
			echo '<p>map_fieldname ('.$tablename.'.'.$fieldname.')</p>';
			var_dump($field);
			//die();
		}
		// Si mapping update
		if (!empty($field['update_fct'])) {
			if (DEBUG_SYNCHRO) {
				echo '<p>field:update_fct : '.$field['update_fct'].'</p>';
				//var_dump($fieldname);
			}
			
			// Analyse
			list($map_tablename, $map_fieldname) = explode('.', $field['update_fct']);
			//var_dump($map_tablename, $map_fieldname);
			
			// Before
			if (false && isset($data[$tablename][$fieldname])) {
				//$u[0][$fieldname] = $a_data[$tablename][$fieldname];
			}
			
			// Fonction de mise à jour
			// Donnée statique préparamétrée
			if ($map_tablename=='val') {
				if (isset(static::$_map_val[$dest][$map_fieldname])) {
					//$u[1][$fieldname] = static::$_map_val[$dest][$map_fieldname];
					$u[$fieldname] = static::$_map_val[$dest][$map_fieldname];
				}
			}
			// Fonctions paramétrées
			elseif ($map_tablename=='fct') {
				if (isset(static::$_map_fct[$dest][$map_fieldname])) {
					$fct = '_'.$dest.'_map_'.$map_fieldname;
					//$u[1][$fieldname] = static::$fct($data_from);
					$u[$fieldname] = static::$fct($data_from, $tree_dest);
				}
			}
			// Mapping de table
			// @todo a termne, plus complexe, utiliser une regex et permettre de combiner par ex: tra{fct.datetime}-{p.label}
			// et utiliser le mapping associé, qui n'est pas collé à la définition du champ
			elseif($map_tablename=='fk') {
				// Foreign key
				if (isset(static::$_map_fk[$map_fieldname])) {
					$fct = '_'.$dest.'_map_fk';
					//$u[1][$fieldname] = static::$fct($data_from);
					$u[$fieldname] = static::$fct($map_fieldname, $data_from, $tree_dest);
				}
				
			}
			elseif(isset(static::${'_'.$orig.'t_alias'}[$map_tablename])) {
				//var_dump($p_diff); die();
				$map_tablename = static::${'_'.$orig.'t_alias'}[$map_tablename];
				// Check champ mis à jour
				if (!empty($field['transform_fct'])) {
					$transform = $field['transform_fct'];
					// @todo voir si on utilise pas plutôt une méthde interne ..? quitte à la mapper direct sur un truc simple, mais au moins c'est plus propre et souple
					//$u[1][$fieldname] = $transform($data_from[$map_tablename][$map_fieldname]);
					$u[$fieldname] = $transform($data_from[$map_tablename][$map_fieldname]);
				}
				else {
					//$u[1][$fieldname] = $data_from[$map_tablename][$map_fieldname];
					//var_dump($data_from);
					$u[$fieldname] = $data_from[$map_tablename][$map_fieldname];
				}
				//echo 'MAPPING - '.$map_tablename.'.'.$map_fieldname.' :'; var_dump($p_data[$map_tablename][$map_fieldname]);
			}
		}
		// Si on est en mode insertion (plus exactement, si on a pas de donnée à mettre à jour)
		elseif (empty($tree_dest) || empty($tree_dest['row'])) {
			if (isset($field['default_fct'])) {
				list($map_tablename, $map_fieldname) = explode('.', $field['default_fct']);
				if ($map_tablename=='val') {
					if (isset(static::$_map_val[$dest][$map_fieldname])) {
						//$u[1][$fieldname] = static::$_map_val[$dest][$map_fieldname];
						$u[$fieldname] = static::$_map_val[$dest][$map_fieldname];
					}
				}
				// Fonctions paramétrées
				if ($map_tablename=='fct') {
					if (isset(static::$_map_fct[$dest][$map_fieldname])) {
						$fct = '_'.$dest.'_map_'.$map_fieldname;
						//$u[1][$fieldname] = static::$fct($data_from);
						$u[$fieldname] = static::$fct($data_from, $tree_dest);
					}
				}
			}
			elseif (!is_null($field['default'])) {
				//$u[1][$fieldname] = $field['default'];
				$u[$fieldname] = $field['default'];
			}
		}
	}
	
	if (false && empty($d_data)) {
		// Gestion dépendance des données si on créé toute la chaine...
		// Par ex, pour un produit décliné, on devrai créer dans l'ordre et utiliser par la suite :
		// - product => id_product
		// - product_shop <= id_product
		// - product_lang <= id_product
		// - product_attribute <= id_product => id_product_attribute
		// - product_attribute_shop <= id_product_attribute
	}
	
	// Clés de modification si on avait une donnée avant
	// Obsolète, on le gère en amont
	if (false && !empty($u[0])) {
		foreach($table->ident_fields() as $fieldname) {
			$u['keys'][$fieldname] = $a_data[$tablename][$fieldname];
		}
	}
		
	return $u;
}


// Object

//protected $id;
protected $_fields = [];
protected $_data = [];

public function __construct()
{
	static::__log(__METHOD__);
	
	//var_dump($this);
}

public function __tostring()
{
	return static::$_label;
}

}
