<?php

namespace ORM;

class Base
{
	
// Meta Class

protected static $__db;
protected static $__logger;

public static function __init()
{
	if (!isset(static::$__logger))
		static::__setlogger($GLOBALS['logger']);
	static::__log(__METHOD__);

	if (!isset(static::$__db))
		static::__setdb($GLOBALS['db']);
}

public static function __setdb($db)
{
	static::$__db = $db;
}

public static function __setlogger($logger)
{
	static::$__logger = $logger;
}

public static function __logger()
{
	return static::$__logger;
}

public static function __log($message, $t='notice')
{
	return static::$__logger->notice($message);
}

// Class

// Meta données

protected static $_db_table;
protected static $_db_id = 'id';
protected static $_db_ref = 'ref';
protected static $_cache = [];

/*
 * Champs du modèle
 */
protected static $_fields = [];

public static function _fields()
{
	return static::$_fields;
}

public static function _field($name)
{
	return static::$_fields[$name];
}

// Manipulation des objets

public static function _get($params)
{
	if (is_numeric($params))
		return static::_get_by_id($params);
	elseif (is_string($params))
		return static::_get_by_name($params);
}

/**
 * Search object by unique id
 */
public static function _get_by_id($id)
{
	static::__log(__METHOD__);
	
	if ($o = static::_cache_get($id))
		return $o;
	
	$db = static::$__db;
	
	// Actual data structure informations
	$sql = static::_select_sql(static::$_db_id.'="'.$id.'"');
	//echo $sql; die();
	$q = $db->query($sql);
	$data = $q->fetch_assoc();
	
	$object = new static($id, $data);
	static::$_cache[$id] = $object;
	
	//static::_get_more_by_id($id);
	
	return $object;
}
//protected static function _get_more_by_id($id, $object);

/**
 * Search object by unique ref/name
 */
public static function _get_by_name($name)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	// Actual structure informations
	$sql = static::_search_sql(static::$_db_ref.'="'.$name.'"');
	//echo $sql;
	$q = $db->query($sql);
	if (!(list($id) = $q->fetch_row()))
		return;
	
	return static::_get_by_id($id);
}

/**
 * Search objects
 * with params
 */
public static function _search($params=[])
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	// Actual data structure informations
	$sql = static::_search_sql($params);
	//echo $sql; die();
	$q = $db->query($sql);
	
	// already listed
	$l = [];
	// to retrieve
	$t = [];
	
	// Récup en cache
	while(list($id) = $q->fetch_row()) {
		if ($o=static::_cache_get($id))
			$l[] = $o;
		else
			$t[] = $id;
	}
	
	// Récup du reste
	foreach($t as $id)
		$l[] = static::_get_by_id($id);
	
	return $l;
}
public static function _select($params=[])
{
	return static::_search($params);
}

/**
 * Search everything
 */
public static function _list()
{
	return static::_search();
}

public static function _count($params=[])
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	$sql = static::_count_sql($params);
	$q = $db->query($sql);
	list($nb) = $q->fetch_row();
	return $nb;
}

public static function _search_sql($params=null)
{
	static::__log(__METHOD__);
	
	$sql = 'SELECT '.static::$_db_id.'
		FROM '.static::$_db_table;
	if (is_string($params))
		$sql .= ' WHERE '.$params;
		
	return $sql;
}

public static function _count_sql($params=null)
{
	static::__log(__METHOD__);
	
	$sql = 'SELECT COUNT(*)
		FROM '.static::$_db_table;
	if (is_string($params))
		$sql .= ' WHERE '.$params;
		
	return $sql;
}

public static function _select_sql($params=null)
{
	static::__log(__METHOD__);
	
	$sql = 'SELECT *
		FROM '.static::$_db_table;
	if (is_string($params))
		$sql .= ' WHERE '.$params;
		
	return $sql;
}

public static function _data_get($id)
{
	
}

public static function _cache_exists($id)
{
	return is_numeric($id) && isset(static::$_cache[$id]);
}

public static function _cache_get($id)
{
	if (is_numeric($id) && isset(static::$_cache[$id]))
		return static::$_cache[$id];
}

public static function _cache_set($object)
{
	static::$_cache[$object->id] = $object;
}


public static function _insert($data)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	// @todo real_escape_string()
	$sql = 'INSERT INTO `'.static::$_db_table.'`
		(`'.implode('`, `', array_keys($data)).'`)
		VALUES ("'.implode('", "', $data).'");';
	//echo '<p>'.$sql.'</p>'; return;
	$q = $db->query($sql);
	//var_dump($q);
	if (! ($id = $db->insert_id))
		return;
	$o = new static($id, $data);
	static::_cache_set($o);
	
	return $o;
}

public static function _replace($data)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	$sql = 'REPLACE INTO `'.static::$_db_table.'`
		(`'.implode('`, `', array_keys($data)).'`)
		VALUES ("'.implode('", "', $data).'");';
	//echo '<p>'.$sql.'</p>'; return;
	$db->query($sql);
}

public static function _update($id, $data)
{
	static::__log(__METHOD__);
	$db = static::$__db;
	
	$u = [];
	foreach($data as $i=>$j)
		$u[] = '`'.$i.'`='.(is_null($j) ?'NULL' :'"'.$j.'"');
	$sql = 'UPDATE `'.static::$_db_table.'`
		SET '.implode(', ', $u).'
		WHERE id = '.$id;
	//echo '<p>'.$sql.'</p>'; return;
	return $db->query($sql);
}

// Object

protected $id;
protected $data = [];

public function __construct($id=null, $data=[])
{
	static::__log(__METHOD__);
	
	$this->id = $id;
	if (!empty($data))
		$this->data_set($data);
}

public function data_set($data)
{
	static::__log(__METHOD__);
	
	if (!is_array($data))
		return;
	foreach($data as $name=>$value) {
		$this->data[$name] = $value;
	}
}

public function id_get()
{
	//static::__log(__METHOD__);
	
	return $this->id;
}

public function data_get()
{
	//static::__log(__METHOD__);
	
	return $this->data;
}

public function __get($name)
{
	if ($name=='id')
		return $this->id;
	return $this->data[$name];
}

public function __isset($name)
{
	return is_string($name) && ($name=='id' || isset($this->data[$name]));
}

public function update($data)
{
	static::__log(__METHOD__);
	
	return static::_update($this->id, $data);
}

public function insert($data)
{
	static::__log(__METHOD__);
	
	return static::_insert($data);
}
	
}

Base::__init();
