<?php

namespace Table;

require_once CLASS_PATH.'/orm/base.php';

class Field extends \ORM\Base {

/* to overload */
protected static $_prefix;
protected static $_db_prefix;
protected static $_cache = [];
protected static $_table_cache = [];
protected static $_c_cache = [];
protected static $_i_cache = [];
protected static $_map_cache = [];
protected static $_ref_cache = [];
protected static $_alias_cache = [];

/*
 * Champs d'une table
 */
protected static $_fields = [
	'id' => [
		'label' => 'ID',
		'type' => 'int',
		'size' => '10',
	],
	'table_id' => [
		'label' => 'Table',
		'type' => 'object',
		'object' => 'table',
	],
	'pos' => [
		'label' => 'Position',
		'type' => 'int',
	],
	'name' => [
		'label' => 'référence',
		'type' => 'string',
		'ref',
	],
	'label' => [
		'label' => 'Label',
		'type' => 'string',
		'label',
	],
	'db_type' => [
		'label' => 'Type BDD',
		'type' => 'string',
	],
	'type' => [
		'label' => 'Type',
		'type' => 'list',
		'list' => [
			'int' => 'Integer',
			'object' => 'Objet',
			'float' => 'Float',
			'bool' => 'Booléen',
			'string' => 'Chaîne de caractères (limitée)',
			'text' => 'Texte (long)',
			'list' => 'Liste de sélection',
			'date' => 'Date',
			'timestamp' => 'Timestamp',
		],
	],
	'size' => [
		'label' => 'Taille champ',
		'type' => 'string',
		'size' => '8',
	],
	'default' => [
		'label' => 'Valeur par défaut (DB)',
		'type' => 'string',
	],
	'default_fct' => [
		'label' => 'Valeur par défaut (fonction à l\'insertion)',
		'type' => 'string',
	],
	'update_fct' => [
		'label' => 'Fonction de mapping pour update',
		'type' => 'string',
	],
	'transform_fct' => [
		'label' => 'Fonction de transformation pour update',
		'type' => 'string',
	],
	'null' => [
		'label' => 'NULL possible',
		'type' => 'bool',
	],
	'options' => [
		'label' => 'Options',
		'type' => 'list',
		'list' => [
			'unsigned'=>'unsigned',
			'zerofill'=>'zerofill',
			'auto_increment'=>'auto_increment',
		],
		'multiple',
	],
	'index' => [
		'label' => 'Index',
		'type' => 'list',
		'list' => [
			'index'=>'Index',
			'unique'=>'Unique',
			'primary'=>'Primary',
		],
	],
	'fk_field' => [
		'label' => 'Clé étrangère',
		'type' => 'string',
	],
	'fk_field_id' => [
		'label' => 'ID Clé étrangère',
		'type' => 'object',
		'object' => 'field',
	],
];

/*
 * Champs définis par la fonction show columns MySQL
 */
protected static $_c_fields = [
	'Field' => [
		'label'=>'Name',
	],
	'Type' => [
		'label'=>'Type',
	],
	'Null' => [
		'label'=>'Is NULL',
	],
	'Key' => [
		'label'=>'Key',
	],
	'Default' => [
		'label'=>'Default value',
	],
	'Extra' => [
		'label'=>'Extra',
	],
	'Comment' => [
		'label'=>'Commentaire',
	],
];

public static function __init()
{
	parent::__init();
	
	static::_cache_all();
}

public static function _cache_all($force=false)
{
	if (!empty(static::$_ref_cache) && !$force)
		return;
	
	// Rows
	$sql = 'SELECT *
		FROM _'.static::$_prefix.'fields';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = static::$__db->query($sql);
	//var_dump($q); die();
	while($row = $q->fetch_assoc()) {
		$o = new static($row['id'], $row);
		static::$_cache[$row['id']] = $o;
		static::$_table_cache[$row['table_id']][$row['name']] = $o;
	}
	
	// Ref/alias, etc.
	$sql = 'SELECT f.id, t.ref, t.alias, f.name
		FROM _'.static::$_prefix.'fields f
		INNER JOIN _'.static::$_prefix.'tables t ON t.id=f.table_id';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = static::$__db->query($sql);
	//var_dump($q); die();
	while($row = $q->fetch_assoc()) {
		static::$_ref_cache[$row['ref']][$row['name']] = $row['id'];
		static::$_alias_cache[$row['alias']][$row['name']] = $row['id'];
	}
}

public static function _ref_to_id($tablename, $fieldname)
{
	if (isset(static::$_ref_cache[$tablename][$fieldname]))
		return static::$_ref_cache[$tablename][$fieldname];
}

public static function _c_fields()
{
	return static::$_c_fields;
}

public static function _c_field($name)
{
	return static::$_c_fields[$name];
}

public static function _c_data_get($table_name)
{
	static::__log(__METHOD__);
	
	if (!empty(static::$_c_cache[$table_name]))
		return static::$_c_cache[$table_name];
	
	$db = static::$__db;
	
	// Table informations
	$sql = 'SHOW FULL COLUMNS FROM '.static::$_db_prefix.$table_name;
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	$f = [];
	while($r=$q->fetch_assoc())
		$f[$r['Field']] = $r;
	
	return static::$_c_cache[$table_name] = $f;
}

public static function _i_data_get($table_name)
{
	static::__log(__METHOD__);
	
	if (!empty(static::$_i_cache[$table_name]))
		return static::$_i_cache[$table_name];
	
	$db = static::$__db;
	
	// Table informations
	$sql = 'SHOW INDEX FROM '.static::$_db_prefix.$table_name;
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	$f = [];
	while($r=$q->fetch_assoc()) {
		unset($r['Cardinality']);
		$f[$r['Key_name']] = $r;
	}
	
	return static::$_i_cache[$table_name] = $f;
}

/**
 * Insert DB Field
 */
public static function _c_insert($table_name, $row)
{
	$options = explode(',', $row['Extra']);
	$sql = 'ALTER TABLE `'.static::$_db_prefix.$table_name.'`
		ADD `'.$row['Field'].'` '.$row['Type'].($row['Collation'] ?' CHARACTER SET utf8mb4'.' COLLATE '.$row['Collation'] :'').($row['Null'] ?' NULL' :' NOT NULL').' DEFAULT '.(is_null($row['Default']) ?'NULL' :'"'.$row['Default'].'"').' '.(in_array('auto_increment', $options) ?' AUTO_INCREMENT' :'').(in_array('unsigned', $options) ?' UNSIGNED' :'').';';
	if (DEBUG_SQL)
		var_dump($sql);
	return static::$__db->query($sql);
}
/**
 * Update DB Field
 */
public static function _c_update($table_name, $row)
{
	$options = explode(',', $row['Extra']);
	$sql = 'ALTER TABLE `'.static::$_db_prefix.$table_name.'`
		CHANGE `'.$row['Field'].'` `'.$row['Field'].'` '.$row['Type'].($row['Collation'] ?' CHARACTER SET utf8mb4'.' COLLATE '.$row['Collation'] :'').($row['Null']=='YES' ?' NULL' :' NOT NULL').' DEFAULT '.(is_null($row['Default']) ?'NULL' :'"'.$row['Default'].'"').' '.(in_array('auto_increment', $options) ?' AUTO_INCREMENT' :'').(in_array('unsigned', $options) ?' UNSIGNED' :'').';';
	if (DEBUG_SQL)
		var_dump($sql);
	return static::$__db->query($sql);
}
/**
 * Delete DB Field
 */
public static function _c_delete($table_name, $row)
{
	$sql = 'ALTER TABLE `'.static::$_db_prefix.$table_name.'`
		DROP column `'.$row['Field'].'`;';
	if (DEBUG_SQL)
		var_dump($sql);
	return static::$__db->query($sql);
}

/**
 * Insert Index
 */
public static function _i_insert($table_name, $row)
{
	$c_data = static::_c_data_get($table_name);
	//var_dump($c_data); die();
	//$options = explode(',', $row['Extra']);
	$sql = 'ALTER TABLE `'.static::$_db_prefix.$table_name.'`
		ADD INDEX `'.$row['Key_name'].'` (`'.$row['Column_name'].'`);';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = static::$__db->query($sql);
	//var_dump($q);
	return $q;
}
/**
 * Update Index
 */
public static function _i_update($table_name, $row)
{
	$c_data = static::_c_data_get($table_name);
	//var_dump($c_data); die();
	//$options = explode(',', $row['Extra']);
	$sql = 'ALTER TABLE `'.static::$_db_prefix.$table_name.'`
		CHANGE INDEX `'.$row['Key_name'].'` (`'.$row['Column_name'].'`);';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = static::$__db->query($sql);
	//var_dump($q);
	return $q;
}
/**
 * Delete Index
 */
public static function _i_delete($table_name, $row)
{
	$c_data = static::_c_data_get($table_name);
	//var_dump($c_data); die();
	//$options = explode(',', $row['Extra']);
	$sql = 'ALTER TABLE `'.static::$_db_prefix.$table_name.'`
		DROP INDEX `'.$row['Key_name'].'`';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = static::$__db->query($sql);
	//var_dump($q);
	return $q;
}

/**
 * Champs d'une table, indexés par le nom du champ
 */
public static function _select_by_table($id)
{
	static::__log(__METHOD__);
	
	return static::$_table_cache[$id];
}

// Object

public function map_data_get($map_id)
{
	if (!isset(static::$_map_cache[$this->id])) {
		// On en a déjà => pas requête
		if (!empty(static::$_map_cache)) {
			static::$_map_cache[$this->id] = [];
		}
		
		else {
			static::$_map_cache[$this->id] = [];
			$sql = 'SELECT *
				FROM _map_'.static::$_prefix.'fields';
			if (DEBUG_SQL)
				var_dump($sql);
			$q = static::$__db->query($sql);
			//var_dump($q);
			while($row = $q->fetch_assoc()) {
				if (false && !empty($row['field_id'])) {
					static::$_map_cache[$row['field_id']][$row['map_id']] = $row;
				}
				elseif(!empty($row['field'])) {
					list($tablealias, $fieldname) = explode('.', $row['field']);
					$field_id = static::$_alias_cache[$tablealias][$fieldname];
					static::$_map_cache[$field_id][$row['map_id']] = $row;
					//var_dump($tablename); var_dump($fieldname);
				}
				else {
					var_dump('COUILLE ASSOC CHAMP MAP');
					continue;
				}
			}
			//var_dump($map); die();
		}
	}
	
	if (isset(static::$_map_cache[$this->id][$map_id]))
		return array_merge($this->data, static::$_map_cache[$this->id][$map_id]);
	else
		return $this->data;
}

}

//Field::__init();

