<?php

namespace Table;

require_once CLASS_PATH.'/orm/base.php';
require_once CLASS_PATH.'/table/field.php';

class Table extends \ORM\Base
{

// Class

protected static $_prefix;

/*
 * Données d'une table
 */
protected static $_fields = [
	'id' => [
		'label'=>'ID',
		'pk',
	],
	'ref' => [
		'label'=>'Nom/Référence',
		'unique',
		'ref',
	],
	'label' => [
		'label'=>'Label',
		'label',
	],
	'alias' => [
		'label'=>'Alias',
		'unique',
	],
	'details' => [
		'label'=>'Details',
	],
	'intl' => [
		'label'=>'Intl',
	],
	'f_pk' => [
		'label'=>'Champ(s) Primary Key',
	],
	'f_uk' => [
		'label'=>'Champ(s) Unique Key',
	],
	'f_fk' => [
		'label'=>'Champs Foreign Key (obsolète, géré via dans table champ)',
	],
	'f_ref' => [
		'label'=>'Champ Réf',
	],
	'f_ts' => [
		'label'=>'Champ Timestamp',
	],
	'f_label' => [
		'label'=>'Champ Label',
	],
	'f_active' => [
		'label'=>'Champ active',
	],
	'f_lang' => [
		'label'=>'Champ lang',
	],
	'f_shop' => [
		'label'=>'Champ Shop',
	],
	'f_parent' => [
		'label'=>'Champ Parent',
	],
];

protected static $_row_cache = [];
protected static $_ref_cache = [];
protected static $_alias_cache = [];

public static function __init()
{
	parent::__init();
	static::_cache_all();
}

public static function _cache_all($force=false)
{
	if (!empty(static::$_alias_cache) && !$force)
		return;
	
	// On met tout en cache direct !
	$sql = 'SELECT *
		FROM _'.static::$_prefix.'tables';
	if (DEBUG_SQL)
		var_dump($sql);
	//echo $sql;
	$q = static::$__db->query($sql);
	//var_dump($q); die();
	while($row = $q->fetch_assoc()) {
		static::$_alias_cache[$row['alias']] = $row['id'];
		static::$_ref_cache[$row['ref']] = $row['id'];
		static::$_row_cache[$row['id']] = $row;
	}
}

public static function _ref_to_id($tablename)
{
	if (isset(static::$_ref_cache[$tablename]))
		return static::$_ref_cache[$tablename];
}

public static function _alias_to_id($tablename)
{
	if (isset(static::$_alias_cache[$tablename]))
		return static::$_alias_cache[$tablename];
}

public static function _get_by_id($id)
{
	static::__log(__METHOD__);
	
	$table = parent::_get_by_id($id);
	
	$table->fields_set(static::_fields_get_by_id($id));

	return $table;
}

/**
 * Récupérer les champs
 */
protected static function _fields_get_by_id($id)
{
	static::__log(__METHOD__);
	
	$classname = '\\Table\\'.static::$_prefix.'field';
	
	return $classname::_select_by_table($id);
}

// Object

protected $fields = [];
protected $pk_fields = [];
protected $uk_fields = [];
protected $fk_fields = [];

public function __construct($id=null, $data=[], $fields=[])
{
	parent::__construct($id, $data);

	if (!empty($fields))
		$this->fields_set($fields);
}

protected function fields_set($fields)
{
	if (!empty($this->fields) || !is_array($fields))
		return;
	
	//var_dump($this->fields); var_dump($fields);
	foreach($fields as $name=>$field) {
		$this->fields[$name] = $field;
		// Primary
		if ($field->index=='primary')
			$this->pk_fields[] = $field->name;
		// Unique
		elseif ($field->index=='unique')
			$this->uk_fields[] = $field->name;
		// Foreign
		if ($field->fk_field_id)
			$this->fk_fields[$field->name] = $field->fk_field_id;
		elseif ($field->fk_field)
			$this->fk_fields[$field->name] = $field->fk_field;
	}
}

/**
 * Returne la clé primaire, comme liste de champs
 */
public function pk_fields()
{
	return $this->pk_fields;
}

/**
 * Returne "la" clé unique, comme liste de champs
 */
public function uk_fields()
{
	return $this->uk_fields;
}

/**
 * Returne les clé étrangères, comme id de champs des autres tables
 */
public function fk_fields()
{
	return $this->fk_fields;
}

public function ident_fields()
{
	return (!empty($this->pk_fields)) ?$this->pk_fields :$this->uk_fields;
}

public function fields()
{
	//var_dump($this->fields);
	return $this->fields;
}

public function field($name)
{
	if (isset($this->fields[$name]))
		return $this->fields[$name];
}

public function field_exists($name)
{
	return isset($this->fields[$name]);
}

public function field_update($data)
{
	//var_dump($data);
	// Update
	if ($field=$this->field($data['name'])) {
		$field->update($data);
	}
	// Insert
	else {
		$data['table_id'] = $this->id;
	
		$classname = '\\Table\\'.static::$_prefix.'field';
		//var_dump($data);
		$o = $classname::_insert($data);
		$this->fields[$data['name']] = $o;
	}
	
	//die();
}


public function map_fields($map_id)
{
	$r = [];
	//var_dump($this->fields);
	foreach($this->fields as $field) {
		$r[$field->name] = $field->map_data_get($map_id);
	}
	//var_dump($r); die();
	return $r;
}


/**
 * Index les champs de BDD manquant
 */
public function fields_index()
{
	$c_fields = $this->c_fields();
	//var_dump($c_fields);
	
	$fields_update = [];
	$pos = 0;
	foreach($c_fields as $ref=>$row) {
		$o = [];
		
		//$key = $row['Key']; //Inutile
		//$comment = $row['Comment']; //Inutile
		//var_dump($ref);
		
		$type_e = explode(' ', $row['Type']);
		if (isset($type_e[1])) {
			if ($type_e[1]=='unsigned')
				$o[] = 'unsigned';
		}
		if (!empty($row['Extra'])) {
			if ($row['Extra']=='on update current_timestamp()') {
				$o[] = 'update_timestamp';
			}
			elseif (in_array($row['Extra'], ['auto_increment', 'unsigned', 'unique', 'primary'])) {
				$o[] = $row['Extra'];
			}
			else {
				echo 'ATTENTION optino non prise en compte dans champ : '; var_dump($row['Extra']);
			}
		}
		preg_match('/^([a-z]*)(\(([a-zA-Z0-9\'.,-]*)\))?$/', $type_e[0], $matches);
		//echo $type_e[0]; var_dump($matches);
		$type = $matches[1];
		
		// Typage
		if (in_array($type, ['tinyint', 'smallint', 'bigint']))
			$type = 'int';
		if (in_array($type, ['tinytext', 'varchar']))
			$type = 'string';
		if (in_array($type, ['enum', 'set']))
			$type = 'list';
		if (in_array($type, ['decimal', 'double']))
			$type = 'float';
		if (in_array($type, ['datetime']))
			$type = 'timestamp';
		
		// Champs à size
		if (in_array($type, ['int', 'string', 'float']))
			$size = isset($matches[3])?$matches[3] :($type=='double' ?'16,4' :NULL);
		else
			$size = NULL;
		
		// Champs à liste
		if (in_array($type, ['list']))
			$list = $matches[3];
		else
			$list = NULL;
				
		$f = [
			'pos' => $pos,
			'name' => $row['Field'],
			//'label' => $row['Field'],
			'db_type' => $row['Type'],
			'type' => $type,
			'size' => $size,
			'list' => $list,
			'default' => $row['Default'],
			'null' => ($row['Null']=='YES') ?1 :0,
			'options' => implode(',', $o),
			//''
		];
		if (! $this->field_exists($row['Field'])) {
			$f['label'] = $row['Field'];
			// @todo Champ index ? Primary, unique, etc. ?
		}
		
		//echo 'Indéxation champ'; var_dump($row); var_dump($f); continue;
		
		// Comparaison / Update / Créaction
		$this->field_update($f);
		//var_dump($f);
		$fields_update[] = $f;
		$pos++;
	}
	//var_dump($fields_update);
}

public function c_fields()
{
	$classname = '\\Table\\'.static::$_prefix.'field';
	
	return $classname::_c_data_get($this->ref);
}

// Manipulation données des tables

public function get($params)
{
	$db = static::$__db;
	
	$sql = $this->select_sql($params);
	if (DEBUG_SQL)
		var_dump($sql);
	
	$q = $db->query($sql);
	if ($row = $q->fetch_assoc())
		return $row;
}

public function select($params)
{
	$db = static::$__db;
	
	$sql = $this->select_sql($params);
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
	$r = [];
	$keys = $this->ident_fields();
	while($row = $q->fetch_assoc()) {
		$k = [];
		foreach($keys as $keyname)
			$k[] = $row[$keyname];
		$r[implode('-', $k)] = $row;
	}
	
	return $r;
}

public function sql_params($params)
{
	if (is_numeric($params)) {
		return '`'.$this->data['f_pk'].'`='.$params;
	}
	else {
		$sql_params = [];
		foreach($params as $name=>$value)
			$sql_params[] = '`'.$name.'`'.(is_null($value) ?' IS NULL' :'="'.static::$__db->real_escape_string($value).'"');
		return implode(' AND ', $sql_params);
	}
}

public function select_sql($params)
{
	$sql = 'SELECT *
		FROM `'.static::$_db_prefix.$this->data['ref'].'`
		WHERE ';
	
	$sql .= $this->sql_params($params);
	return $sql;
}

public function data_insert($data, $params=[])
{
	$db = static::$__db;
	$sql = $this->data_insert_sql($data, $params);
	if (DEBUG_SQL)
		var_dump($sql);
	//$q = $db->query($sql);
	//$this->log($params, 'i', $data);
}
/**
 * @param $data donnée à insérer
 * @param $params données supplémentaires servant de paramètres
 */
public function data_insert_sql($data, $params=[])
{
	//@todo : vérif si is_numeric => pk ?
	if (!is_array($params))
		$params = [];
	//echo '<p>Création</p>'; var_dump($data);
	$f = $v = [];
	foreach(array_merge($data, $params) as $fieldname=>$value) {
		$f[] = '`'.$fieldname.'`';
		$v[] = (is_null($value) || (empty($value) && $fieldname==$this->f_pk)) ?'NULL' :'"'.static::$__db->real_escape_string($value).'"';
	}
	$sql = 'INSERT INTO `'.static::$_db_prefix.$this->ref.'` ('.implode(', ', $f).') VALUES ('.implode(', ', $v).')';
	return $sql;
}

public function data_delete($params)
{
	$db = static::$__db;
	$sql = $this->data_delete_sql($params);
	if (DEBUG_SQL)
		var_dump($sql);
	//$q = $db->query($sql);
	//$this->log($params, 'd');
}
public function data_delete_sql($params)
{
	//echo '<p>Suppression</p>'; var_dump($params);
	$sql = 'DELETE FROM `'.static::$_db_prefix.$this->ref.'` WHERE '.$this->sql_params($params);
	return $sql;
}

public function data_update($data, $params)
{
	$db = static::$__db;
	$sql = $this->data_update_sql($data, $params);
	//$q = $db->query($sql);
	//$this->log($params, 'u', $data);
}
public function data_update_sql($data, $params)
{
	//echo '<p>Update</p>'; var_dump($data); var_dump($params);
	$u = [];
	foreach($data as $fieldname=>$value)
		$u[] = '`'.$fieldname.'`='.(is_null($value) ?'NULL' :'"'.static::$__db->real_escape_string($value).'"');
	$sql = 'UPDATE `'.static::$_db_prefix.$this->ref.'` SET '.implode(',', $u).' WHERE '.$this->sql_params($params);
	return $sql;
}

/**
 * Log update
 */
public function log($id, $action, $data=null)
{
	$db = static::$__db;
	$rev = 0;
	$rev++;
	// @todo rev & oid
	$sql = 'INSERT INTO `_'.static::$_prefix.'log`
		(`ts`, `tid`, `oid`, `rev`, `action`, `fields`, `details`)
		VALUES
		(NOW(), '.$this->id.', '.$id.', '.$rev.', "'.$action.'", "'.json_encode(is_array($data) ?array_keys($data) :NULL).'", "'.json_encode($data).'")';
	if (DEBUG_SQL)
		var_dump($sql);
	$q = $db->query($sql);
}

}

//Table::__init();

