<?php

// -- DATABASE --

class DB
{

public static function __init()
{
	
}

public static function string_escape($str)
{
	global $db;
	return $db->real_escape_string($str);
}

protected static $_error_log = [];

public static function _error_log()
{
	return static::$_error_log;
}
public static function _debug_format($srv, $db, $sql, $q)
{
	if ($db->error && DEBUG_AFF)
		var_dump($db->error);
	return [
		'server' => $srv,
		'type' => 'query',
		'sql' => htmlentities($sql),
		'affected_rows' => $db->affected_rows,
		'insert_id' => $db->insert_id,
		'error' => $db->error,
	];
}
public static function _debug_query($srv, $db, $sql, $q)
{
	return static::$_error_log[] = static::_debug_format($srv, $db, $sql, $q);
}
public static function _debug_select($srv, $db, $sql, $q)
{
	$error_log = static::_debug_format($srv, $db, $sql, $q);
	$error_log['num_rows'] = $q->num_rows;
	$error_log['type'] = 'select';
	return static::$_error_log[] = $error_log;
}
public static function _debug_count($srv, $db, $sql, $q, $nb)
{
	$error_log = static::_debug_format($srv, $db, $sql, $q);
	$error_log['nb'] = $nb;
	$error_log['type'] = 'count';
	return static::$_error_log[] = $error_log;
}
public static function _debug_insert($srv, $db, $sql, $q)
{
	$error_log = static::_debug_format($srv, $db, $sql, $q);
	$error_log['type'] = 'insert';
	return static::$_error_log[] = $error_log;
}
public static function _debug_update($srv, $db, $sql, $q)
{
	$error_log = static::_debug_format($srv, $db, $sql, $q);
	$error_log['type'] = 'update';
	return static::$_error_log[] = $error_log;
}
public static function _debug_delete($srv, $db, $sql, $q)
{
	$error_log = static::_debug_format($srv, $db, $sql, $q);
	$error_log['type'] = 'delete';
	return static::$_error_log[] = $error_log;
}

/**
 * management tables
 */
public static function o_query($sql)
{
	global $db;
	$q = $db->query($sql);
	if (DEBUG_AFF)
		static::_debug_query('O', $db, $sql, $q);
	return $q;
}
public static function o_select($sql)
{
	global $db;
	$q = $db->query($sql);
	if (DEBUG_AFF)
		static::_debug_select('O', $db, $sql, $q);
	return $q;
}
public static function o_count($sql)
{
	global $db;
	$q = $db->query($sql);
	list($nb) = $q->fetch_row();
	if (DEBUG_AFF)
		static::_debug_count('O', $db, $sql, $q, $nb);
	return $nb;
}
public static function o_insert($sql)
{
	global $db;
	$q = $db->query($sql);
	if (DEBUG_AFF)
		static::_debug_insert('O', $db, $sql, $q);
	if ($q)
		return $db->insert_id;
}
public static function o_update($sql)
{
	global $db;
	$q = $db->query($sql);
	if (DEBUG_AFF)
		static::_debug_update('O', $db, $sql, $q);
	if ($q)
		return $db->affected_rows;
}
public static function o_delete($sql)
{
	global $db;
	$q = $db->query($sql);
	if (DEBUG_AFF)
		static::_debug_delete('O', $db, $sql, $q);
	if ($q)
		return $db->affected_rows;
}

// -- D QUERY SQL --

public static function d_query($sql)
{
	global $db_d;

	$q = $db_d->query($sql);

	if (DEBUG_AFF)
		static::_debug_query('D', $db_d, $sql, $q);
	return $q;
}
public static function d_select($sql)
{
	global $db_d;

	$q = $db_d->query($sql);

	if (DEBUG_AFF)
		static::_debug_select('D', $db_d, $sql, $q);
	return $q;
}
public static function d_count($sql)
{
	global $db_d;
	$q = $db_d->query($sql);
	list($nb) = $q->fetch_row();
	if (DEBUG_AFF)
		static::_debug_count('D', $db_d, $sql, $q, $nb);
	return $nb;
}
public static function d_insert($sql)
{
	global $db_d;
	$q = $db_d->query($sql);
	if (DEBUG_AFF)
		var_dump($q);
	if (DEBUG_AFF)
		static::_debug_insert('D', $db_d, $sql, $q);
	if ($q)
		return $db_d->insert_id;
}
public static function d_update($sql)
{
	global $db_d;
	$q = $db_d->query($sql);
	if (DEBUG_AFF)
		static::_debug_update('D', $db_d, $sql, $q);
	if ($q)
		return $db_d->affected_rows;
}
public static function d_delete($sql)
{
	global $db_d;
	$q = $db_d->query($sql);
	if (DEBUG_AFF)
		static::_debug_delete('D', $db_d, $sql, $q);
	if ($q)
		return $db_d->affected_rows;
}

/**
 * d_get
 */
public static function d_get_rows($tablename, $params, $orderby=NULL, $limit=NULL)
{
	$sql = static::d_get_sql($tablename, $params, $orderby, $limit);
	$q = static::d_select($sql);

	$r = [];
	while ($row=$q->fetch_assoc())
		$r[] = $row;
	return $r;
}
public static function d_get_row($tablename, $params, $orderby=NULL)
{
	$sql = static::d_get_sql($tablename, $params, $orderby, 1);
	$q = static::d_select($sql);
	return $q->fetch_assoc();
}
public static function d_get_sql($tablename, $params, $orderby=NULL, $limit=NULL)
{
	$sql = 'SELECT *
		FROM `'.DB_D_PREFIX.$tablename.'`
		WHERE ';
	if (is_numeric($params)) {
		$sql .= 'rowid='.$params;
	}
	elseif (is_array($params)) {
		$sql_params = [];
		foreach($params as $i=>$j) {
			if (is_array($j)) {
				//@todo espace, NULL, etc.
				$sql_params[] = '`'.$i.'` IN ("'.implode('", "', $j).'")';
			}
			else {
				$sql_params[] = '`'.$i.'`="'.$j.'"';
			}
		}
		$sql .= implode(' AND ', $sql_params);
	}
	elseif (is_string($params)) {
		$sql .= $params;
	}
	
	if ($orderby)
		$sql .= ' ORDER BY '.$orderby;
	
	if ($limit)
		$sql .= ' LIMIT '.$limit;

	return $sql;
}

/**
 * d_insert
 */
public static function d_insert_row($tablename, $data)
{
	$sql = static::d_insert_sql($tablename, $data);
	return static::d_insert($sql);
}
public static function d_insert_sql($tablename, $data)
{
	$sql = 'INSERT INTO `'.DB_D_PREFIX.$tablename.'` ';
	$f = [];
	$v = [];
	foreach($data as $i=>$j) {
		$f[] = '`'.$i.'`';
		$v[] = !is_null($j) ?'"'.addslashes($j).'"' :'NULL';
	}
	$sql .= '('.implode(', ', $f).') VALUES ('.implode(', ', $v).')';

	return $sql;
}

/**
 * d_delete
 */
public static function d_delete_row($tablename, $params)
{
	$sql = static::d_delete_sql($tablename, $params);
	return static::d_delete($sql);
}
public static function d_delete_sql($tablename, $params)
{
	$sql = 'DELETE FROM `'.DB_D_PREFIX.$tablename.'` ';
	if (is_numeric($params)) {
		$sql .= ' WHERE `rowid`='.$params;
	}
	elseif (is_array($params)) {
		$u = [];
		foreach($params as $i=>$j) {
			$u[] = '`'.$i.'`'. (!is_null($j) ?'="'.addslashes($j).'"' :'IS NULL');
		}
		$sql .= ' WHERE ('.implode(' AND ', $u).')';
	}
	else { // NADA par défaut
		$sql .= ' WHERE FALSE';
	}

	return $sql;
}

/**
 * d_update
 */
public static function d_update_row($tablename, $data, $params, $data_orig=[])
{
	// Limit to updated data
	if (!empty($data_orig)) {
		foreach($data as $i=>$j) {
			// Pas censé être là...
			if (!array_key_exists($i, $data_orig)) {
				Throw new Exception('DB field name '.$i.' does not exists in '.$tablename);
				unset($data[$i]);
			}
			elseif($j===$data_orig[$i]) {
				unset($data[$i]);
				unset($data_orig[$i]);
			}
			elseif(is_numeric($j) && is_numeric($data_orig[$i]) && ((float)$j==(float)$data_orig[$i] || round($j, 8)==round($data_orig[$i], 8))) {
				unset($data[$i]);
				unset($data_orig[$i]);
			}
		}
		// Remove useless orig data
		foreach($data_orig as $i=>&$j)
			if (!array_key_exists($i, $data))
				unset($data_orig[$i]);
	}

	if (empty($data))
		return;

	$sql = static::d_update_sql($tablename, $data, $params);
	if ($r = static::d_update($sql)) {
		if (isset($params['rowid'])) {
			$sql = 'INSERT INTO `_d_log`
				(`tid`, `oid`, `rev`, `action`, `fields`, `details`, `orig`)
				SELECT `id`, "'.$params['rowid'].'", "0", "u", "'.addslashes(json_encode(array_keys($data))).'", "'.addslashes(json_encode(array_values($data))).'", "'.addslashes(json_encode(array_values($data_orig))).'"
				FROM `_d_tables`
				WHERE `ref`="'.$tablename.'"';
			//echo $sql;
			static::o_insert($sql);
		}
	}
	return $r;
}
public static function d_update_sql($tablename, $data, $params)
{
	$sql = 'UPDATE `'.DB_D_PREFIX.$tablename.'` ';
	$u = [];
	foreach($data as $i=>$j) {
		$u[] = '`'.$i.'`='.(!is_null($j) ?'"'.addslashes($j).'"' :'NULL');
	}
	$w = [];
	foreach($params as $i=>$j) {
		$w[] = '`'.$i.'`'.(!is_null($j) ?'="'.addslashes($j).'"' :' IS NULL');
	}
	$sql .= 'SET '.implode(', ', $u).' WHERE '.implode(' AND ', $w);

	return $sql;
}

// -- P QUERY SQL --

public static function p_query($sql)
{
	global $db_p;

	$q = $db_p->query($sql);

	if (DEBUG_AFF)
		static::_debug_query('P', $db_p, $sql, $q);
	return $q;
}
public static function p_select($sql)
{
	global $db_p;

	$q = $db_p->query($sql);

	if (DEBUG_AFF)
		static::_debug_select('P', $db_p, $sql, $q);
	return $q;
}
public static function p_count($sql)
{
	global $db_p;
	$q = $db_p->query($sql);
	list($nb) = $q->fetch_row();
	if (DEBUG_AFF)
		static::_debug_count('P', $db_p, $sql, $q, $nb);
	return $nb;
}
public static function p_insert($sql)
{
	global $db_p;

	$q = $db_p->query($sql);

	if (DEBUG_AFF)
		static::_debug_insert('P', $db_p, $sql, $q);
	if ($q)
		return $db_p->insert_id;
}
public static function p_update($sql)
{
	global $db_p;

	$q = $db_p->query($sql);

	if (DEBUG_AFF)
		static::_debug_update('P', $db_p, $sql, $q);
	if ($q)
		return $db_p->affected_rows;
}
public static function p_delete($sql)
{
	global $db_p;

	$q = $db_p->query($sql);

	if (DEBUG_AFF)
		static::_debug_delete('P', $db_p, $sql, $q);
	if ($q)
		return $db_p->affected_rows;
}

/**
 * p_get
 */
public static function p_get_rows($tablename, $params, $orderby=NULL, $limit=NULL)
{
	$sql = static::p_get_sql($tablename, $params, $orderby, $limit);
	$q = static::p_select($sql);

	$r = [];
	while ($row=$q->fetch_assoc())
		$r[] = $row;
	return $r;
}
public static function p_get_row($tablename, $params, $orderby=NULL)
{
	$sql = static::p_get_sql($tablename, $params, $orderby, 1);
	$q = static::p_select($sql);
	return $q->fetch_assoc();
}
public static function p_get_sql($tablename, $params, $orderby=NULL, $limit=NULL)
{
	$sql = 'SELECT *
		FROM `'.DB_P_PREFIX.$tablename.'`
		WHERE ';
	if (is_numeric($params)) {
		$sql .= 'id='.$params;
	}
	elseif (is_array($params)) {
		$sql_params = [];
		foreach($params as $i=>$j)
			$sql_params[] = '`'.$i.'`="'.$j.'"';
		$sql .= implode(' AND ', $sql_params);
	}
	elseif (is_string($params)) {
		$sql .= $params;
	}
	
	if ($orderby)
		$sql .= ' ORDER BY '.$orderby;
	
	if ($limit)
		$sql .= ' LIMIT '.$limit;

	return $sql;
}

/**
 * p_insert
 */
public static function p_insert_row($tablename, $data)
{
	$sql = static::p_insert_sql($tablename, $data);
	return static::p_insert($sql);
}
public static function p_insert_sql($tablename, $data)
{
	$sql = 'INSERT INTO `'.DB_P_PREFIX.$tablename.'` ';
	$f = [];
	$v = [];
	foreach($data as $i=>$j) {
		$f[] = '`'.$i.'`';
		$v[] = !is_null($j) ?'"'.addslashes($j).'"' :'NULL';
	}
	$sql .= '('.implode(', ', $f).') VALUES ('.implode(', ', $v).')';

	return $sql;
}

/**
 * p_delete
 */
public static function p_delete_row($tablename, $params)
{
	$sql = static::p_delete_sql($tablename, $params);
	return static::p_delete($sql);
}
public static function p_delete_sql($tablename, $params)
{
	$sql = 'DELETE FROM `'.DB_P_PREFIX.$tablename.'` ';
	if (is_array($params)) {
		$u = [];
		foreach($params as $i=>$j) {
			$u[] = '`'.$i.'`'. (!is_null($j) ?'="'.addslashes($j).'"' :'IS NULL');
		}
		$sql .= ' WHERE ('.implode(' AND ', $u).')';
	}
	else { // NADA par défaut
		$sql .= ' WHERE FALSE';
	}

	return $sql;
}

/**
 * p_update
 */
public static function p_update_row($tablename, $data, $params, $data_orig=[])
{
	// Limit to updated data
	if (!empty($data_orig)) {
		foreach($data as $i=>&$j) {
			if($j===$data_orig[$i] || (is_numeric($j) && is_numeric($data_orig[$i]) && $j==$data_orig[$i])) {
				unset($data[$i]);
				unset($data_orig[$i]);
			}
		}
		// Remove useless orig data
		foreach($data_orig as $i=>&$j)
			if (!array_key_exists($i, $data))
				unset($data_orig[$i]);
	}

	if (empty($data))
		return;

	//@todo : encore $data and $data_orig with int key using $fields
	$sql = static::p_update_sql($tablename, $data, $params);
	if ($r = static::p_update($sql)) {
		if (isset(sync::P_TABLE_ID[$tablename]))
			$fieldname = sync::P_TABLE_ID[$tablename];
		elseif (isset(sync::P_TABLE_ID2[$tablename]))
			$fieldname = sync::P_TABLE_ID2[$tablename];
		else
			$fieldname = NULL;
		if ($fieldname && isset($params[$fieldname])) {
			$sql = 'INSERT INTO `_p_log`
				(`tid`, `oid`, `rev`, `action`, `fields`, `details`, `orig`)
				SELECT `id`, "'.$params[$fieldname].'", "0", "u", "'.addslashes(json_encode(array_keys($data))).'", "'.addslashes(json_encode(array_values($data))).'", "'.addslashes(json_encode(array_values($data_orig))).'"
				FROM `_p_tables`
				WHERE `ref`="'.$tablename.'"';
			//echo $sql;
			static::o_insert($sql);
		}
	}
	return $r;

}
public static function p_update_sql($tablename, $data, $params)
{
	$sql = 'UPDATE `'.DB_P_PREFIX.$tablename.'` ';
	$u = [];
	foreach($data as $i=>$j) {
		$u[] = '`'.$i.'`='.(!is_null($j) ?'"'.addslashes($j).'"' :'NULL');
	}
	$w = [];
	foreach($params as $i=>$j) {
		$w[] = '`'.$i.'`'.(!is_null($j) ?'="'.addslashes($j).'"' :' IS NULL');
	}
	$sql .= 'SET '.implode(', ', $u).' WHERE '.implode(' AND ', $w);

	return $sql;
}

}

DB::__init();
