<?php

/**
 * Erreur de synchro
 * 
 * @param String $localisation fichier/ligne/méthode
 * @param String $message Information/Explication courte
 * @param [] $context Contexte objet
 * @param Throwable $e Exception
 * @param String $notif_ts Raison pour non resynchronisation, il faut le qualifier au maximum, afin de soulager le processus de resynchro, ça ne sert à rien de resynchroniser à l'infini un truc qui merdouille
 */
function sync_error($localisation, $message, $context=[], $e=NULL, $notif_ts=NULL)
{
	static $nb = 0;
	static $notified = false;
	static $notify = [];
	
	$nb++;
	if (DEBUG_AFF) {
		echo '<hr /><p>ERROR '.$nb.' :</p>'
			.'<p>WHERE : '.$localisation.'</p>'
			.'<p>MESSAGE : '.$message.'</p>';
	}
	
	// Si on a un contexte de mise à jour
	if (!empty($context)) { // ['type'=>static::MODEL_NAME, 'sens'=>'pd', 'action'=>'osync', 'otype'=>$otype, 'oid'=>$oid]
		// @todo if empty($contexte['required_field'])

		if (!isset($context['type']))
			$context['type'] = '';
		if (!isset($context['otype']))
			$context['otype'] = '';
		if (!isset($context['oid']))
			$context['oid'] = '';

		if (in_array($context['action'], ['delete', 'odelete']))
			$sql_action = '`action` IN ("delete", "odelete")';
		elseif (in_array($context['action'], ['osync', 'create']))
			$sql_action = '`action` IN ("osync", "create")';
		else
			$sql_action = "FALSE";
		
		// Dernière demande de synchro en BDD non traitée
		$sql = 'SELECT id, user_id, sync_errors
			FROM `_sync`
			WHERE type="'.$context['type'].'" AND t_name="'.$context['otype'].'" AND t_oid='.$context['oid'].'
				AND '.$sql_action.'
				AND sync_ts IS NULL
			ORDER BY id DESC
			LIMIT 1';
		
		if($context['sens']=='pd')
			$q = DB::p_select($sql);
		elseif ($context['sens']=='dp')
			$q = DB::d_select($sql);
		//var_dump($q->num_rows);
		
		// Requête OK => contexte validé
		if (!empty($q)) {
			$context_hash = $context['type'].'-'.$context['otype'].'-'.$context['oid'];
			if ($q->num_rows) {
				// On incrémente le nbre d'erreurs de la dernière synchro non achevée
				list($sync_id, $user_id, $sync_errors) = $q->fetch_row();
				if($user_id) {
					if ($context['sens']=='pd')
						list($email) = DB::p_select('SELECT email FROM '.DB_P_PREFIX.'employee WHERE id_employee='.$user_id)->fetch_row();
					elseif ($context['sens']=='dp')
						list($email) = DB::d_select('SELECT email FROM '.DB_D_PREFIX.'user WHERE rowid='.$user_id)->fetch_row();
				}

				$sql = 'UPDATE _sync
					SET sync_errors=sync_errors+1
					WHERE id='.$sync_id;
				if($context['sens']=='pd')
					DB::p_update($sql);
				elseif ($context['sens']=='dp')
					DB::d_update($sql);
				
				// Raison+ts => on note pourquoi on resynchronise pas
				if (is_string($notif_ts)) {
					// Dernier en date
					$sql = 'UPDATE `_sync`
						SET `sync_ts`=NOW(), `sync_result`="'.$notif_ts.'"
						WHERE `type`="'.$context['type'].'" AND `t_name`="'.$context['otype'].'" AND `t_oid`="'.$context['oid'].'"
							AND '.$sql_action.'
							AND `sync_ts` IS NULL
						ORDER BY id DESC
						LIMIT 1';
					if($context['sens']=='pd')
						DB::p_update($sql);
					elseif ($context['sens']=='dp')
						DB::d_update($sql);
					// Ce qui reste (avant)
					$sql = 'UPDATE `_sync`
						SET `sync_ts`=NOW(), `sync_result`="cascade"
						WHERE `type`="'.$context['type'].'" AND `t_name`="'.$context['otype'].'" AND `t_oid`="'.$context['oid'].'"
							AND '.$sql_action.'
							AND `sync_ts` IS NULL';
					if($context['sens']=='pd')
						DB::p_update($sql);
					elseif ($context['sens']=='dp')
						DB::d_update($sql);
				}
				
				// Notify l'ensemble des erreurs des objets pour lesquels on a une demande de synchro fraiche
				// Demande fraiche
				if ($sync_errors==0)
					$notify[] = $context_hash;
				// Demande pas fraiche
				elseif (!in_array($context_hash, $notify)) {
					echo '<p>'.$context_hash.' NOT IN NOTIFY TABLE => on zappe on va pas plus loin</p>';
					var_dump($e);
					return;
				}
			}
			// Contexte sans résultat
			// Si on a déjà traité et validé avec ts, on envoie l'erreur, sinon on affiche une anomalie
			elseif (in_array($context_hash, $notify)) {
				notify('Pas/Plus d\'enregistrement _sync de demande mise à jour trouvé pour la donnée... :<br />'.$sql.'<br />Context : '.json_encode($context));
			}
			else {
				notify('ATTENTION! ANOMALIE? Est-ce un test manuel ? Pas d\'enregistrement _sync de demande mise à jour trouvé pour la donnée... :<br />'.$sql.'<br />Context : '.json_encode($context));
				//return;
			}
		}
		// Contexte sans résultat
		else {
			notify('<b>ATTENTION! ANOMALIE! requête _sync de demande mise à jour en erreur pour la donnée... :</b><br />'.$sql.'<br />Context : '.json_encode($context));
		}
		// Une seule fois par objet
		if (!isset($sync_errors) || $sync_errors==0) {
			notify('<b>ERREUR SYNCHO '.$context['action'].' '.($context['sens']=='pd' ?'Presta=>Doli' :'Doli=>Presta').' : objet '.($context['sens']=='pd' ?'Presta' :'Doli').' de type '.$context['type'].'/'.$context['otype'].' ID: '.$context['oid'].' vers objet '.($context['sens']=='pd' ?'Doli' :'Presta').'</b>'
				.($e ?' :<br />'.$e->getmessage() :'')
				.'<br /><b><a href="https://'.$_SERVER['SERVER_NAME'].'/sync.php?sens='.$context['sens'].'&type='.$context['type'].'&otype='.$context['otype'].'&oid='.$context['oid'].'&action='.$context['action'].'&password='.WS_PASSWORD.'">Relancer la synchro</a></b>');
		}
	}
	// Envoi de la notification
	//echo '<p>ENVOI NOTIF MAIL : '.$localisation.'<br />'.$message.'</p>';
	if (!empty($email))
		notify('Utilisateur concerné : '.$email, NULL, $email);
	notify($localisation.'<br />'.$message, $e);
	//notify($localisation.'<br />'.$message, $e, !empty($email) ?$email :NULL);
	$notified = true;

	// DB log
	if (DEBUG_AFF) {
		echo '<hr /><h3>DB error log</h3>';
		var_dump(DB::_error_log());
	}
}

function notify($message, $e=NULL, $email=NULL)
{
	static $notify;
	if (empty($notify))
		$notify = new notify();
	
	$message = '<p>'.$message.'</p>';
	if ($e) {
		$dump = ob_var_dump($e);
		$message .= '<br /><div style="max-height: 400px;overflow: auto;">'.$dump.'</div>';
	}

	$notify->add($message);

	if (!empty($email))
		$notify->email_add($email);
}

function ob_var_dump($e)
{
	ob_start();
	var_dump($e);
	$dump = ob_get_contents();
	ob_end_clean();
	
	return $dump;
}

function pre_dump($var)
{
	echo '<pre>';
	var_dump($var);
	echo '</pre>';
}

function pre_echo($var)
{
	echo '<pre>'.$var.'</pre>';
}

function pre_sql($sql)
{
	pre_echo($sql);
}

function gettopost()
{
	foreach($_GET as $i=>&$j) {
		if (!isset($_POST[$i]))
			$_POST[$i] = $j;
	}
}

function auth()
{
	$r = !empty($_POST['password']) && $_POST['password']===WS_PASSWORD;
	if(isset($_POST['password']))
		unset($_POST['password']);
	if(isset($_GET['password']))
		unset($_GET['password']);
	return $r;
}

function replace_accents($str)
{
	return iconv('UTF-8', 'ASCII//TRANSLIT', $str); 
}

function code_compta($code_compta, $max=8)
{
	return preg_replace('/([^a-z0-9])/i', '', replace_accents(
		strtoupper(substr($code_compta, 0, $max))
	));
}
