<?php
/**
* 2021 Mathieu Moulin iProspective
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    Mathieu Moulin <contact@iprospective.fr>
*  @copyright 2021 Mathieu Moulin iProspective
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of iProspective & Mathieu Moulin
*/

if (!defined('_PS_VERSION_')) {
	exit;
}

//use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use Symfony\Component\Validator\Constraints\Url;
use Symfony\Component\Validator\Validation;

/**
 * Module Version Maj.Min.Debug
 * - Major = Big changes
 * - Minor = Little changes
 * - Debug = Same API but bugs corrected
 *
 */

class MMI_Module_1_0 extends Module
{
	const MMI_MODULE_VERSION = '1.0.0';
	
	const TPL_ADMIN = '';

	const PARAMETERS_PREFIX = '';
	const PARAMETERS = [];
	
	const TRANS_PREFIX = '';
	
	const FOLDER_PREFIX = '';
	
	const HOOKS = [];
	
	// Infos
	const NAME = '';
	const TAB = '';
	const VERSION = '1.0.0';
	const IS_CONFIGURABLE = 1;
	const NEED_INSTANCE = 0;
	const AUTHOR = 'Mathieu Moulin iProspective';
	const BOOTSTRAP = true;
	const DISPLAY_NAME = '';
	const DESCRIPTION = '';
	const UNINSTALL_CONFIRM = 'Are you sure to uninstall ?';
	const MODULE_KEY = '';
	const VERSION_PS_MIN = '1.7';
	
	public function __construct()
	{
		$this->name = static::NAME;
		$this->tab = static::TAB;
		$this->version = static::VERSION;
		$this->is_configurable = static::IS_CONFIGURABLE;
		$this->need_instance = static::NEED_INSTANCE;
		$this->author = static::AUTHOR;

		$this->bootstrap = static::BOOTSTRAP;
		
		if (static::MODULE_KEY)
			$this->module_key = static::MODULE_KEY;

		$this->displayName = $this->trans(static::DISPLAY_NAME, array(), 'Modules.'.static::TRANS_PREFIX.'.Admin');
		$this->description = $this->trans(static::DESCRIPTION, array(), 'Modules.'.static::TRANS_PREFIX.'.Admin');

		$this->confirmUninstall = $this->trans(static::UNINSTALL_CONFIRM, array(), 'Modules.'.static::TRANS_PREFIX.'.Admin');

		$this->ps_versions_compliancy = array('min' => static::VERSION_PS_MIN, 'max' => _PS_VERSION_);
	
		parent::__construct();
	}
	
	/* INSTALL */
	
	public function registerHooks()
	{
		foreach(static::HOOKS as $hookName) {
			if (!$this->registerHook($hookName))
				return false;
		}
		return true;
	}
	
	// @todo unregisterHooks ?

	public function install()
	{
		foreach (static::PARAMETERS as $name=>$param) {
			Configuration::updateValue(static::PARAMETERS_PREFIX.strtoupper($name), '');
		}
		
		include(dirname(__FILE__).'/sql/install.php');

		return parent::install()
			&& $this->registerHooks();
	}

	public function uninstall()
	{
		foreach (static::PARAMETERS as $name=>$param) {
			Configuration::deleteByName(static::PARAMETERS_PREFIX.strtoupper($name));
		}
		
		include(dirname(__FILE__).'/sql/uninstall.php');

		return parent::uninstall();
	}
	
	/* CONFIG */

	public function getContent()
	{
		if (Tools::isSubmit('submitModule')) {
			$this->updateFields();
			$this->_clearCache('*');

			Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&tab_module='.$this->tab.'&conf=4&module_name='.$this->name);
		}

		return $this->renderForm();
	}
	

	public function _clearCache($template, $cache_id = null, $compile_id = null)
	{
		parent::_clearCache(static::TPL_ADMIN);
	}

	public function renderForm()
	{
		$fields_form = array(
			'form' => array(
				'legend' => array(
					'title' => $this->trans('Settings', array(), 'Admin.Global'),
					'icon' => 'icon-cogs'
				),
				'input' => array(

				),
				'submit' => array(
					'title' => $this->trans('Save', array(), 'Admin.Global'),
				)
			),
		);

		foreach (static::PARAMETERS as $name=>$param) {
			$field = [
				'id' => 'mmi_param_'.$name,
				'type' => $param['form_type'],
				'label' => $this->trans($param['label'], array(), 'Modules.'.static::TRANS_PREFIX.'.Admin'),
				'name' => $name,
				'desc' => $this->trans($param['desc'], array(), 'Modules.'.static::TRANS_PREFIX.'.Admin'),
			];
			if (isset($param['value']))
				$field['value'] = $param['value'];
			if (isset($param['values']))
				$field['values'] = $param['values'];

			$fields_form['form']['input'][] = $field;
		}

		$helper = new HelperForm();
		$helper->show_toolbar = false;
		$helper->table =  $this->table;
		$helper->submit_action = 'submitModule';
		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFieldsValues(),
		);

		return $helper->generateForm(array($fields_form));
	}

	public function getConfigFieldsValue($name)
	{
		return Configuration::get(static::PARAMETERS_PREFIX.strtoupper($name));
	}

	public function getConfigFieldsValues()
	{
		$result = [];
		foreach (static::PARAMETERS as $name=>$param) {
			$result[$name] = Configuration::get(static::PARAMETERS_PREFIX.strtoupper($name));
		}
		return $result;
	}

	/**
	* Update form fields.
	* Check all social networks form value and verify the URL is valid.
	* Do nothing if a violation is spotted.
	*/
	protected function updateFields()
	{
		$validator = Validation::createValidator();

		foreach (static::PARAMETERS as $name=>$param) {
			$value = Tools::getValue($name, '');
			if (!empty($param['sf_constraint'])) {
				$constraint = $param['sf_constraint'];
				if ($constraint=='Url')
					$constraints = [new Url()];
				$violations = $validator->validate($value, $constraints);
				if (0 === count($violations)) {
					Configuration::updateValue(static::PARAMETERS_PREFIX.strtoupper($name), $value);
				}
			}
			else {
				Configuration::updateValue(static::PARAMETERS_PREFIX.strtoupper($name), $value);
			}
		}
	}
}

