<?php
/**
* 2021 Mathieu Moulin iProspective
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    Mathieu Moulin <contact@iprospective.fr>
*  @copyright 2021 Mathieu Moulin iProspective
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of iProspective & Mathieu Moulin
*/

class WS
{
	
protected static $_type = 'p';
protected static $_db;
protected static $_db_prefix = 'ps_';
protected static $_params = [
	'id_shop' => 1,
	'id_lang' => 1,
	'lang' => 'fr',
];

public function __construct($params=null)
{
	if (is_array($params)) {
		static::$_params = $params;
	}
	if (empty(static::$_db)) {
		$p = static::_parameters();
		static::$_db = new mysqli($p['database_host'], $p['database_user'], $p['database_password'], $p['database_name']);
	}
}

// Prestashop
public static function _parameters()
{
	$a = require_once '../../app/config/parameters.php';
	return $a['parameters'];
}

public static function _gettopost($name)
{
	if (!isset($_POST[$name]) && isset($_GET[$name]))
		$_POST[$name] = $_GET[$name];
}

public static function _gettopostall()
{
	foreach($_GET as $name=>$value)
		if (!isset($_POST[$name]) && isset($_GET[$name]))
			$_POST[$name] = $_GET[$name];
}

protected $error = [];

protected static $_actions = [
	'schema_fields' => [],
	'schema_index' => [],
	'notif' => [],
	'exists' => [],
	'get' => [],
	'update' => [],
	'sql' => [],
];

// Ici les modèles sont en fait des mappings


protected static $_tables = [
	// Produit simple
	'product' => [
		'pk' => 'id_product',
	],
	'product_lang' => [
		'uk' => [
			'id_product',
			'id_lang',
			'id_shop',
		],
		'fk' => [
			'product' => 'id_product',
			'lang' => 'id_lang',
			'shop' => 'id_shop',
		],
	],
	'product_shop' => [
		'uk' => [
			'id_product',
			'id_shop',
		],
		'fk' => [
			'product' => 'id_product',
			'shop' => 'id_shop',
		],
	],
	'product_supplier' => [
		'pk' => 'id_product_supplier',
		'fk' => [
			'product' => 'id_product',
			'supplier' => 'id_supplier',
		],
	],
	'stock_available' => [
		'pk' => 'id_stock_available',
		'fk' => [
			'product' => 'id_product',
			'shop' => 'id_shop',
		],
	],
	'products_dlc_dluo' => [
		'pk' => 'id',
		'fk' => [
			'product' => 'id_product',
			'warehouse' => 'id_warehouse',
		],
	],
	'product_attribute' => [
		'pk' => 'id_product_attribute',
		'fk' => [
			'product' => 'id_product',
		],
	],
	/*
	'product_attribute_combination' => [
		'uk' => [
			'id_product_attribute',
			'id_attribute',
		],
		'fk' => [
			'product_attribute' => 'id_product_attribute',
			'attribute' => 'id_product',
		],
	],
	*/
	'product_attribute_shop' => [
		'uk' => [
			'id_product_attribute',
			'id_shop',
		],
		'fk' => [
			'product' => 'id_product',
			'lang' => 'id_lang',
			'shop' => 'id_shop',
		],
	],
//	'product_attribute_image' => [],
	'product_lang' => [
		'uk' => [
			'id_product',
			'id_lang',
			'id_shop',
		],
		'fk' => [
			'product' => 'id_product',
			'lang' => 'id_lang',
			'shop' => 'id_shop',
		],
	],
	'supplier' => [
		'pk' => 'id_supplier',
	],
	'supplier_lang' => [
		'uk' => [
			'id_supplier',
			'id_lang',
		],
		'fk' => [
			'supplier' => 'id_supplier',
			'lang' => 'id_lang',
		],
	],
	'supplier_shop' => [
		'uk' => [
			'id_supplier',
			'id_shop',
		],
		'fk' => [
			'supplier' => 'id_supplier',
			'shop' => 'id_shop',
		],
	],
	'address' => [
		'pk' => 'id_address',
		'fk' => [
			'supplier' => 'id_supplier',
			'customer' => 'id_customer',
		],
	],
	'customer' => [
		'pk' => 'id_customer',
	],
	'orders' => [
		'pk' => 'id_order',
	],
	'order_detail' => [
		'pk' => 'id_order_detail',
		'uk' => [],
		'fk' => [
			'order' => 'id_order',
			'product' => 'id_product',
			'product_attribute' => 'id_product_attriibute',
		],
	],
];

protected static $_maps = [
	// Produit simple
	1 => [
		'product' => [
			'get' => 'row',
		],
		'product_lang' => [
			'get' => 'row',
			//'get' => 'list',
		],
		'product_shop' => [
			'get' => 'row',
			//'get' => 'list',
		],
		'product_supplier' => [
			'get' => 'list',
		],
		'stock_available' => [
			'get' => 'row',
		],
		'products_dlc_dluo' => [
			'get' => 'list',
		],
	],
	// Produit décliné
	2 => [
		'product' => [
			'get' => 'row',
		],
		'product_attribute' => [
			'get' => 'row',
		],
		'product_attribute_combination' => [
			'get' => 'list',
		],
		'product_attribute_shop' => [
			'get' => 'row',
		],
	//	'product_attribute_image' => [],
		'product_lang' => [
			'get' => 'row',
			//'get' => 'list',
		],
		'product_shop' => [
			'get' => 'row',
			//'get' => 'list',
		],
		'product_supplier' => [
			'get' => 'list',
		],
		'stock_available' => [
			'get' => 'row',
		],
		'products_dlc_dluo' => [
			'get' => 'list',
		],
	],
	// Fournisseur
	3 => [
		'supplier' => [
			'get' => 'row',
		],
		'supplier_lang' => [
			'get' => 'row',
			//'get' => 'list',
		],
		'supplier_shop' => [
			'get' => 'row',
			//'get' => 'list',
		],
		'address' => [
			'get' => 'list',
			//'get' => 'row',
			'uk' => [
				'id_supplier',
				'alias',
			],
		],
	],
	// Client
	4 => [
		'customer' => [
			'get' => 'row',
		],
		'address' => [
			'get' => 'list',
			//'get' => 'row',
			'uk' => [
				'id_customer',
				'alias',
			],
		],
	],
	// Order
	8 => [
		'orders' => [
			'get' => 'row',
		],
		'order_detail' => [
			'get' => 'list',
			//'get' => 'row',
		],
	],
//	'specific_price' => [], // Toute une série
//	'product_sale' => [],
//	etc.
];


public function authenticate()
{
	//$_POST['passkey'];
	//echo 'Authenticated OK';
	return true;
}

public function execute()
{
	if (!empty($_GET['action']))
		$this->action($_GET['action']);
}

public function action($action)
{
	if (empty($action) || !array_key_exists($action, static::$_actions))
		die('Action needed');
	//echo 'Action : '.$action;
	
	$this->{$action.'_action'}();
}

// Actions

// Schéma
public function schema_fields_action()
{
	static::_gettopost('tablename');
	
	if (empty($_POST['tablename']))
		die(json_encode(['error'=>'Table name required']));
	
	$data = static::_c_data_get($_POST['tablename']);
	
	die(json_encode($data, true));
}
public function schema_index_action()
{
	static::_gettopost('tablename');
	
	if (empty($_POST['tablename']))
		die(json_encode(['error'=>'Table name required']));
	
	$data = static::_c_index_get($_POST['tablename']);
	
	die(json_encode($data, true));
}
public static function _c_data_get($table_name)
{
	$db = static::$_db;
	
	// Table informations
	$sql = 'SHOW FULL COLUMNS FROM '.static::$_db_prefix.$table_name;
	//echo $sql; die();
	$q = $db->query($sql);
	$f = [];
	while($r=$q->fetch_assoc())
		$f[$r['Field']] = $r;
	
	return $f;
}
public static function _c_index_get($table_name)
{
	$db = static::$_db;
	
	// Table informations
	$sql = 'SHOW INDEX FROM '.static::$_db_prefix.$table_name;
	//echo $sql; die();
	$q = $db->query($sql);
	$f = [];
	while($r=$q->fetch_assoc()) {
		unset($r['Cardinality']);
		$f[$r['Key_name']] = $r;
	}
	
	return $f;
}

// Envoi modifications
public function notif_action()
{
	$sql = "SELECT DISTINCT t_id, oid, operation, MAX(ts)
		FROM _log
		WHERE notif_ts IS NULL
		GROUP BY t_id, oid, operation";
	
	// Récupération données à mettre à jour
	
	// Order
	// Payment
	// Product
	// Category
}

// Mise à jour
public function update_action()
{
	static::_gettopost('list');
	
	if (empty($_POST['list']))
		die(json_encode(['error'=>'List required']));
	var_dump($_POST['list']);
}


// Vérif existence objet
public function exists_action()
{
	static::_gettopost('tablename');
	static::_gettopost('id');
	
	if (empty($_POST['tablename']))
		die(json_encode(['error'=>'tablename required']));
	if (!is_string($tablename=$_POST['tablename']) || !isset(static::$_tables[$tablename]))
		die(json_encode(['error'=>'tablename invalid']));
	$table = static::$_tables[$tablename];
	if (empty($table['pk']))
		die(json_encode(['error'=>'tablename without pk']));
	if (empty($_POST['id']))
		die(json_encode(['error'=>'id required']));
	if (!is_numeric($id=$_POST['id']))
		die(json_encode(['error'=>'id invalid']));
	
	$sql = 'SELECT *
		FROM `'.static::$_db_prefix.$tablename.'`
		WHERE `'.$table['pk'].'`='.$id;
	$q = static::$_db->query($sql);
	if ($row=$q->fetch_row())
		$data = ['return'=>1, 'row'=>$row];
	else
		$data = ['return'=>0, 'reason'=>'row id '.$id.' does not exists'];
	die(json_encode($data));
}

// Récupération objet
public function get_action()
{
	static::_gettopost('map');
	static::_gettopost('id');
	
	if (empty($_POST['map']))
		die(json_encode(['error'=>'map required']));
	if (empty($_POST['id']))
		die(json_encode(['error'=>'id required']));
	
	$map = $_POST['map'][static::$_type.'t'][$_POST['map'][static::$_type.'t_main']];
	$data = static::_get_data_tree($map, $map['name'], $_POST['id']);
	//echo 'grrr'; print_r($data); die();
	die(json_encode($data));
}

public function sql_action()
{
	static::_gettopost('sql');
	static::_gettopost('type');
	
	if (empty($_POST['sql']))
		die(json_encode(['error'=>'type required']));
	
	$db = static::$_db;
	
	// @todo vérifier et retourner si erreur
	if (is_string($_POST['sql'])) {
		$q = $db->query($_POST['sql']);
	}
	elseif(is_array($_POST['sql'])) {
		$q = [];
		foreach($_POST['sql'] as $sql) {
			$q[] = $db->query($sql);
		}
	}
	else {
		$q = null;
	}
	
	$r = ['response'=>true, 'q'=>$q, 'id'=>$db->insert_id];
	
	die(json_encode($r));
}

// Manipulations modèle


/**
 * ATTENTION! : params doit être construit en amont
 */
public static function _get_data_tree($map, $tablename, $params)
{
	// Récupération données de base
	$datainit = [
		'tablename' => $tablename,
		'params' => $params,
		'join_type' => $map['join_type'],
		'children' => [],
	];
	
	if ($map['join_type']=='row') {
		//echo '<p>'.$tablename.'</p>';
		//var_dump(array_keys(static::${'_'.$type.'t'}));
		$row = static::_table_get($map, $tablename, $params);
		return static::_get_data_tree_sub($map, $datainit, $row);
	}
	else {
		$rows = static::_table_get($map, $tablename, $params);
		$datas = [];
		foreach($rows as $key=>$row) {
			$datas[$key] = static::_get_data_tree_sub($map, $datainit, $row);
		}
		return $datas;
	}
}
/**
 * Sous-requête
 * Juste histoire de factoriser et simplifier le code
 */
public static function _get_data_tree_sub($map, $data, $row)
{
	$data['row'] = $row;
	// récursion enfants, si on a quelque chose ! sinon ça sert à rien on pourra pas requêter...
	if (!is_null($row) && !empty($map['children'])) foreach($map['children'] as $tname=>$t) {
		//echo '<p>'.$tname.'</p>'; var_dump($t);// die();
		$params = [
			$t['join']['fieldname'] => $row[$t['join']['from_fieldname']],
		];
		if (!empty($t['params'])) {
			$tparams = explode(',', $t['params']);
			foreach($tparams as $i)
				$params[$i] = static::$_params[$i];
		}
		$data['children'][$tname] = static::_get_data_tree($t, $tname, $params);
	}
	
	return $data;
}

public static function _table_get($map, $tablename, $params)
{
	//var_dump($map); die();
	$map_id = $map['map_id'];
	$table = array_merge(static::$_tables[$tablename], static::$_maps[$map_id][$tablename]);
	//var_dump($table); die();
	
	if (isset($table['pk']) && is_numeric($params)) {
		$id = $params;
		$sql = 'SELECT * FROM `'.static::$_db_prefix.$tablename.'` WHERE '.$table['pk'].'='.$id;
		$q = static::$_db->query($sql);
	}
	elseif (isset($table['fk']) && is_array($params)) {
		$sql = 'SELECT * FROM `'.static::$_db_prefix.$tablename.'` WHERE ';
		$sql_params = [];
		foreach($params as $name=>$value)
			$sql_params[] = '`'.$name.'`='.$value;
		$sql .= implode(' AND ', $sql_params);
		$q = static::$_db->query($sql);
	}
	//return '<p>SQL : '.$sql.'</p>'; //die();
		
	// Renvoi row ou liste
	if ($table['get']=='row') {
		if ($row = $q->fetch_assoc()) {
			//var_dump($row);
			return $row;
		}
	}
	elseif ($table['get']=='list') {
		$keys = null;
		if (isset($table['pk']))
			$keys = [$table['pk']];
		elseif (isset($table['uk']))
			$keys = $table['uk'];
		$r = [];
		while($row = $q->fetch_assoc()) {
			if (!empty($keys)) {
				$k = [];
				foreach($keys as $keyname)
					$k[] = $row[$keyname];
				$r[implode('-', $k)] = $row;
			}
			else {
				$r[] = $row;
			}
		}
		return $r;
	}
}

}
