<?php
/**
* 2021 Mathieu Moulin iProspective
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    Mathieu Moulin <contact@iprospective.fr>
*  @copyright 2021 Mathieu Moulin iProspective
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of iProspective & Mathieu Moulin
*/

if (!defined('_PS_VERSION_')) {
	exit;
}

//use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use Symfony\Component\Validator\Constraints\Url;
use Symfony\Component\Validator\Validation;

class MMI_Product extends Module
{
    const TPL_ADMIN = 'module:mmi_product/views/templates/admin/config.tpl';

    const PARAMETERS_PREFIX = 'MMI_PRODUCT_';
    const PARAMETERS = [
		'miniature_cache'=>[
			'type'=>'bool',
			'form_type'=>'checkbox',
			'value'=>"1",
			//'sf_constraint'=>'Url',
			'label'=>'Cache miniature',
			'desc'=> 'Nettoyage du cache de miniature lorsque modification image produit',
		],
		'compose_promo_check'=>[
			'type'=>'bool',
			'form_type'=>'checkbox',
			'value'=>"1",
			//'sf_constraint'=>'Url',
			'label'=>'Gestion produits composés',
			'desc'=> 'Vérification si remise sur composant de produit => alerte email',
		],
	];
	
	const TRANS_PREFIX = 'MMIProduct';
	
	const FOLDER_PREFIX = 'mmi_product';
	
	CONST HOOKS = [
		'actionProductSave',
	];
	
	public function __construct()
	{
		$this->name = 'mmi_product';
		$this->tab = 'administration';
		$this->version = '1.0.0';
		$this->author = 'Mathieu Moulin iProspective';
		$this->is_configurable = 1;
		$this->need_instance = 0;

		$this->bootstrap = true;

		parent::__construct();

		$this->displayName = $this->trans('Amélioration de la gestion des produits', array(), 'Modules.'.self::TRANS_PREFIX.'.Admin');
		$this->description = $this->trans('Gestion des doublons, etc.', array(), 'Modules.'.self::TRANS_PREFIX.'.Admin');

		//$this->confirmUninstall = $this->trans('Uninstall confirmation..?');

		$this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
	}
	
	/* INSTALL */
	
	public function registerHooks()
	{
		foreach(self::HOOKS as $hookName) {
			if (!$this->registerHook($hookName))
				return false;
		}
		return true;
	}

	public function install()
	{
        foreach (static::PARAMETERS as $name=>$param) {
			Configuration::updateValue(self::PARAMETERS_PREFIX.strtoupper($name), '');
		}
		include(dirname(__FILE__).'/sql/install.php');

		return parent::install()
			&& $this->registerHooks();
	}

	public function uninstall()
	{
        foreach (static::PARAMETERS as $name=>$param) {
			Configuration::deleteByName(self::PARAMETERS_PREFIX.strtoupper($name));
		}
		
		include(dirname(__FILE__).'/sql/uninstall.php');

		return parent::uninstall();
	}
	
	/* CONFIG */

	public function getContent()
	{
		if (Tools::isSubmit('submitModule')) {
			$this->updateFields();
			$this->_clearCache('*');

			Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules').'&configure='.$this->name.'&tab_module='.$this->tab.'&conf=4&module_name='.$this->name);
		}

		return $this->renderForm();
	}
	

    public function _clearCache($template, $cache_id = null, $compile_id = null)
    {
        parent::_clearCache(self::TPL_ADMIN);
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->trans('Settings', array(), 'Admin.Global'),
                    'icon' => 'icon-cogs'
                ),
                'input' => array(
                    
                ),
                'submit' => array(
                    'title' => $this->trans('Save', array(), 'Admin.Global'),
                )
            ),
        );
	
        foreach (static::PARAMETERS as $name=>$param) {
			$field = [
				'id' => 'mmi_param_'.$name,
				'type' => $param['form_type'],
				'label' => $this->trans($param['label'], array(), 'Modules.'.self::TRANS_PREFIX.'.Admin'),
				'name' => $name,
				'desc' => $this->trans($param['desc'], array(), 'Modules.'.self::TRANS_PREFIX.'.Admin'),
			];
			if (isset($param['value']))
				$field['value'] = $param['value'];
			if (isset($param['values']))
				$field['values'] = $param['values'];

			$fields_form['form']['input'][] = $field;
		}
		
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table =  $this->table;
        $helper->submit_action = 'submitModule';
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues()
    {
        $result = [];
        foreach (static::PARAMETERS as $name=>$param) {
            $result[$name] = Configuration::get(self::PARAMETERS_PREFIX.strtoupper($name));
        }
        return $result;
    }

    /**
     * Update form fields.
     * Check all social networks form value and verify the URL is valid.
     * Do nothing if a violation is spotted.
     */
    protected function updateFields()
    {
        $validator = Validation::createValidator();

        foreach (static::PARAMETERS as $name=>$param) {
			$value = Tools::getValue($name, '');
			if (!empty($param['sf_constraint'])) {
				$constraint = $param['sf_constraint'];
				if ($constraint=='Url')
					$constraints = [new Url()];
				$violations = $validator->validate($value, $constraints);
				if (0 === count($violations)) {
					Configuration::updateValue(self::PARAMETERS_PREFIX.strtoupper($name), $value);
				}
			}
			else {
				Configuration::updateValue(self::PARAMETERS_PREFIX.strtoupper($name), $value);
			}
        }
    }
	
	/* HOOKS */

	public function hookActionProductSave($param)
	{
		//var_dump($param); die();
		$config = $this->getConfigFieldsValues();
		
		if (!(
			is_array($param)
			&& is_numeric($id_product = $param['id_product'])
		))
			return;
		
		//var_dump($config); die();
		
		// Suppression Thumbnails
		if (true || !empty($config['thumbnail_clean']))
			$this->thumbnail_clean($id_product);
		
		// Check produit composé promo
		if (true || !empty($config['compose_promo_check']))
			$this->compose_promo_check($id_product);
	}
	
	// Suppression Thumbnails
	public function thumbnail_clean($id_product)
	{
		foreach (glob(_PS_IMG_DIR_.'tmp/product_mini_'.$id_product.'*') as $filename) {
			unlink($filename);
		}
	}
	
	// Check produit composé promo
	public function compose_promo_check($id_product)
	{
		$db = Db::getInstance();
		$sql = 'SELECT id_product_pack
			FROM ps_pack
			WHERE id_product_item = '.$id_product;
		$result = $db->executes($sql);
		$list = '';
		foreach($result as $row) {
			//var_dump($row);
			$list .= 'Pack produit n.'.$row['id_product_pack']."\r\n";
		}
		$email = 'sandrine@calicote.com';
		$email = 'mathieu@iprospective.fr';
		mail($email, 'Produit remisé dans un pack', 'Produit n.'.$id_product."\r\n".$list, 'From: no-reply@calicote.com');
		//die();
	}
	
}
