<?php
/**
* 2021 Mathieu Moulin iProspective
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    Mathieu Moulin <contact@iprospective.fr>
*  @copyright 2021 Mathieu Moulin iProspective
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of iProspective & Mathieu Moulin
*/

if (!defined('_PS_VERSION_')) {
	exit;
}

//use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use Symfony\Component\Validator\Constraints\Url;
use Symfony\Component\Validator\Validation;

// Use autoload ..?
if (! class_exists('MMI_Module_1_0'))
	require_once 'classes/mmi_module.php';
if (! class_exists('MMI_Notification_1_0'))
	require_once 'classes/mmi_notification.php';

class MMI_Sync extends MMI_Module_1_0
{
	const TPL_ADMIN = 'module:mmi_sync/views/templates/admin/config.tpl';

	const PARAMETERS_PREFIX = 'MMI_SYNC_';
	const PARAMETERS = [
		'sync_url'=>[
			'type'=>'text',
			'form_type'=>'text',
			'value'=>1,
			//'sf_constraint'=>'Url',
			'label'=>'URL Synchronisation',
			'desc'=> 'URL du webservice de synchro à notifier',
		],
		'ws_key'=>[
			'type'=>'text',
			'form_type'=>'text',
			'value'=>1,
			//'sf_constraint'=>'Url',
			'label'=>'Clé Webservice',
			'desc'=> 'Clé pour accéder au webservice local',
		],
	];
	
	const TRANS_PREFIX = 'MMISync';
	
	const FOLDER_PREFIX = 'mmi_sync';
	
	const HOOKS = [
		// Product
		'actionObjectProductUpdateAfter',
		'actionObjectProductDeleteAfter',
		'actionObjectProductAddAfter',
		// Supplier
		'actionObjectSupplierUpdateAfter',
		'actionObjectSupplierDeleteAfter',
		'actionObjectSupplierAddAfter',
		// Customer
		'actionObjectCustomerUpdateAfter',
		'actionObjectCustomerDeleteAfter',
		'actionObjectCustomerAddAfter',
		// Order
		'actionObjectOrderUpdateAfter',
		'actionObjectOrderDeleteAfter',
		'actionObjectOrderAddAfter',
	];
	
	// Info
	const NAME = 'mmi_sync';
	const TAB = 'administration';
	const VERSION = '1.0.0';
	const IS_CONFIGURABLE = 1;
	const NEED_INSTANCE = 0;
	const BOOTSTRAP = true;
	const DISPLAY_NAME = 'Synchronisation ERP Dolibarr';
	const DESCRIPTION = 'Produits, commandes, BL, Factures, Stock, etc.';
	const MODULE_KEY = '';
	const VERSION_PS_MIN = '1.7';
	
	/* Webservice */

	/* HOOKS */
	
	public function ws_trigger($type, $otype, $action, $param)
	{
		if (empty($param) || !isset($param['object']) || !is_object($object=$param['object']) || !isset($object->id) || !is_numeric($id=$object->id))
			return;
		
		//var_dump($object->id); die();
		
		$ref = $otype.'-'.$id.'-'.$action;
		MMI_Notification_1_0::_getInstance()->add('sync', $type, $ref, ['otype'=>$otype, 'oid'=>$id, 'action'=>$action]);
		//var_dump($type); var_dump($ref); die();
		return true;
		// Notif
	}

	// Product
	public function hookActionObjectProductAddAfter($param)
	{
		$this->ws_trigger('product', 'product', 'create', $param);
	}
	public function hookActionObjectProductUpdateAfter($param)
	{
		$this->ws_trigger('product', 'product', 'osync', $param);
	}
	public function hookActionObjectProductDeleteAfter($param)
	{
		$this->ws_trigger('product', 'product', 'odelete', $param);
	}

	// Supplier
	public function hookActionObjectSupplierAddAfter($param)
	{
		$this->ws_trigger('supplier', 'supplier', 'create', $param);
	}
	public function hookActionObjectSupplierUpdateAfter($param)
	{
		$this->ws_trigger('supplier', 'supplier', 'osync', $param);
	}
	public function hookActionObjectSupplierDeleteAfter($param)
	{
		$this->ws_trigger('supplier', 'supplier', 'odelete', $param);
	}

	// Customer
	public function hookActionObjectCustomerAddAfter($param)
	{
		$this->ws_trigger('customer', 'customer', 'create', $param);
	}
	public function hookActionObjectCustomerUpdateAfter($param)
	{
		$this->ws_trigger('customer', 'customer', 'osync', $param);
	}
	public function hookActionObjectCustomerDeleteAfter($param)
	{
		$this->ws_trigger('customer', 'customer', 'odelete', $param);
	}

	// Order
	public function hookActionObjectOrderAddAfter($param)
	{
		$this->ws_trigger('order', 'orders', 'create', $param);
	}
	public function hookActionObjectOrderUpdateAfter($param)
	{
		$this->ws_trigger('order', 'orders', 'osync', $param);
	}
	public function hookActionObjectOrderDeleteAfter($param)
	{
		$this->ws_trigger('order', 'orders', 'odelete', $param);
	}
}
