<?php

class sync_payment_pd extends sync_payment
{

	public static function pd_check_sync(&$o, &$p_data)
	{
		// Uniquement si on active la synchro et que la date est ok
		if(empty(PAYMENT_SYNC) || empty(PAYMENT_DATE_BEGIN_SYNC) || empty($p_data['order_payment']['date_add']) || $p_data['order_payment']['date_add'] < PAYMENT_DATE_BEGIN_SYNC)
			return false;

		// Uniquement les paiements CBI et PAYPAL, pas les virements ni chèques, qui seront enregistrés proprement à la main sur DOLI
		$type = static::_pd_map_type_paiement($p_data);
		if (!in_array($type, ['CBI', 'PAYPAL']))
			return false;
		
		return true;
	}

// -- SUB ACTIONS --

public function pd_insert_more(&$o, &$d_data_map, &$p_data)
{
	// Un seul cas de $otype/$d_tref dans Doli
	$id = $o['d_oid'];
	
	// Payment associated to commande
	$d_data_map['paiement_object']['fk_paiement'] = $id;
	DB::d_insert_row('paiement_object', $d_data_map['paiement_object']);

	// Payment associated to invoice
	static::_d_api('GET', 'mmipaymentapi/commande_payment_invoice_assoc/'.$d_data_map['paiement_object']['fk_object']);
}

// -- MAPPING --

public function pd_map_update(&$o, &$p_data, &$d_data=[])
{
	// No update. Notified once
	//return;
	
	$d_data_map = [
		'paiement' => [
			'fk_paiement' => static::_pd_map_fk_paiement($p_data),
		]
	];

	return $d_data_map;
}

public function pd_map_create(&$o, &$p_data)
{
	// E-Transactions
	if (substr($p_data['order_payment']['payment_method'], 0, 14)=='E-Transactions') {
		if (!empty($p_data['etransactions_order'])) {
			$moreinfo = ' id_transaction '.$p_data['etransactions_order']['id_transaction'];
			$amount = $p_data['etransactions_order']['amount']/100;
			$date = $p_data['etransactions_order']['date'];
		}
		else {
			$moreinfo = ' infos de transaction manquantes...';
			$amount = $p_data['order_payment']['amount'];
			$date = $p_data['order_payment']['date_add'];
		}
	}
	// Up2pay e-Transactions Crédit Agricole
	elseif (substr($p_data['order_payment']['payment_method'], 0, 21)=='Up2pay e-Transactions') {
		if (!empty($p_data['up2pay_deadlines'])) {
			$moreinfo = ' id_transaction '.$p_data['up2pay_transaction']['numtrans'];
			// 1st effective transaction
			foreach($p_data['up2pay_deadlines'] as $deadline) {
				if (! $deadline['captured'])
					continue;
				$amount = $deadline['amount'];
				$date = $deadline['date_execution'];
				break;
			}
		}
		elseif (!empty($p_data['up2pay_transaction'])) {
			$moreinfo = ' id_transaction '.$p_data['up2pay_transaction']['numtrans'];
			$amount = $p_data['up2pay_transaction']['amount'];
			$date = $p_data['up2pay_transaction']['date_add'];
		}
		else {
			$moreinfo = ' infos de transaction manquantes...';
			$amount = $p_data['order_payment']['amount'];
			$date = $p_data['order_payment']['date_add'];
		}
	}
	// Prestashop Checkout + Paypal
	elseif ($p_data['order_payment']['payment_method']=='PrestaShop Checkout') {
		if (!empty($p_data['pscheckout_order_matrice']))
			$moreinfo = ' id_transaction '.$p_data['pscheckout_order_matrice']['id_order_paypal'];
		else
			$moreinfo = ' infos de transaction manquantes...';
		$amount = $p_data['order_payment']['amount'];
		$date = $p_data['order_payment']['date_add'];
	}
	// Prestashop Checkout + Paypal
	elseif (is_numeric(strpos('PayPal', $p_data['order_payment']['payment_method']))) {
		if (!empty($p_data['paypal_order'])) {
			$moreinfo = ' id_transaction '.$p_data['paypal_order']['id_transaction'];
			$amount = $p_data['paypal_order']['total_paid'];
			$date = $p_data['paypal_order']['date_add'];
		}
		else {
			$moreinfo = ' infos de transaction manquantes...';
			$amount = $p_data['order_payment']['amount'];
			$date = $p_data['order_payment']['date_add'];
		}
	}
	else {
		$moreinfo = '';
		$amount = $p_data['order_payment']['amount'];
		$date = $p_data['order_payment']['date_add'];
	}
	
	$fk_object = static::_pd_map_fk('order', 'orders', $p_data['orders']['id_order']);
	$tms = substr($date, 4,4).'-'.substr($date, 2,2).'-'.substr($date, 0,2).' 00:00:00';

	$d_data_map = [
		'paiement' => [
			//'rowid' => '',
			'ref' => static::d_ref(), // PAY0000-0000 // @todo ajouter nom du client en paramètre pour certaines configurations !
			'entity' => 1, // @todo est-ce toujours ainsi ..? à vérifier
			'datec' => $p_data['order_payment']['date_add'],
			'tms' => static::$datetime,
			'datep' => $p_data['order_payment']['date_add'],
			'amount' => $amount,
			'multicurrency_amount' => $amount,
			'fk_paiement' => static::_pd_map_fk_paiement($p_data),
			'num_paiement' => NULL,
			'note' => 'Prestashop order '.$p_data['order_payment']['order_reference'].$moreinfo,
			'ext_payment_id' => NULL,
			'ext_payment_site' => NULL,
			'fk_bank' => 0, // Numéro d'écriture en bancaire, sera ajouté sur Dolibarr
			'fk_user_creat' => static::D_K_USER,
			'fk_user_modif' => static::D_K_USER,
			'statut' => 0,
			'fk_export_compta' => 0,
		],
		'paiement_extrafields' => [
			//'fk_object' => '',
		],
		'paiement_object' => [
			//'fk_paiement' => '',
			'objecttype' => 'Commande',
			'fk_object' => $fk_object,
			'amount' => $amount,
			'multicurrency_code' => '',
			'multicurrency_tx' => 1,
			'multicurrency_amount' => $amount,
		],
	];
	
	if (false && substr($p_data['order_payment']['payment_method'], 0, 14)=='E-Transactions' && !empty($p_data['etransactions_order'])) {
		$d_data_map['mbi_etransactions_hash'] = [
			'objecttype' => "Commande",
			'fk_object' => $fk_object,
			'amount' => $p_data['etransactions_order']['amount'],
			'multiple' => $p_data['order_payment']['payment_method']=='E-TransactionsRecurring' ?'3' : NULL,
			'hash' => $p_data['etransactions_order']['num_appel'], // ..?
			'tms' => $tms,
			'info' => NULL,
		];
		$d_data_map['mbi_etransactions_return'] = [
			//'fk_mbi_etransactions' => ,
			'tms' => $tms,
			'mt' => $p_data['etransactions_order']['amount'],
			'auto' => $p_data['etransactions_order']['num_appel'],
			'erreur' => '00000',
			'trans' => $p_data['etransactions_order']['id_transaction'],
			//'fk_paiement' => ,
		];
		// $p_data['etransactions_order']['num_appel']
		// $p_data['etransactions_order']['currency'] = 978 (une seule valeur) => €
		// $p_data['etransactions_order']['payment_status'] = capture (une seule valeur)
		// $p_data['etransactions_order']['payment_by'] = E-Transactions / E-TransactionsRecurring
		// $p_data['etransactions_order']['method'] = 3DSECURE / PAYPAL / CARTE / WALLET /
		// $p_data['etransactions_order']['carte'] = CB / PAYPAL / AMEX / VISA / E8CARD / Mastercard / EUROCARD_MASTERCARD
		// $p_data['etransactions_order']['carte_num'] = 
		// $p_data['etransactions_order']['pays'] = FRA, LUX, etc.
		// $p_data['etransactions_order']['ip'] = FRA, etc.
		// $p_data['etransactions_order']['secure'] = O/N
		// $p_data['etransactions_order']['refund_amount'] = 0
	}
	
	if (false) {
		$d_data_map['paiement_facture'] = [
			//'rowid' => NULL,
			//'fk_paiement' => , => rowid
			'fk_facture' => '',
			'amount' => $amount,
			'multicurrency_code' => NULL,
			'multicurrency_tx' => '1',
			'multicurrency_amount' => $amount,
		];
	}

	return $d_data_map;
}

// -----------

// Field MAPPING

public static function _pd_map_type_paiement(&$orig_data, &$dest_tree=[])
{
	switch($orig_data['order_payment']['payment_method']) {
		case 'E-Transactions':
		case 'E-Transactions (x3)':
		case 'Up2pay e-Transactions Crédit Agricole':
			$type = 'CBI';
			break;
		case 'CB TEL':
		case 'TPE':
			$type = 'CB';
			break;
		case 'Transfert bancaire':
			$type = 'VIR';
			break;
		case 'Chèque':
		case 'Cheque':
			$type = 'CHQ';
			break;
		case 'Espèces':
		case 'Especes':
		case 'Caisse':
			$type = 'LIQ';
			break;
		case 'PrestaShop Checkout':
		case 'PayPal':
			$type = 'PAYPAL';
			break;
		default:
			$type = 'UNDEF';
	}
	return $type;
}

public static function _pd_map_fk_paiement(&$orig_data, &$dest_tree=[])
{
	$type = static::_pd_map_type_paiement($orig_data, $dest_tree);
	return static::$_d_c_paiement[$type]['id'];
}

public static function _pd_map_fk_bank(&$orig_data, &$dest_tree=[])
{
	switch($orig_data['order_payment']['payment_method']) {
		case 'E-Transactions':
		case 'E-Transactions (x3)':
		case 'Up2pay e-Transactions Crédit Agricole':
		case 'PayPal':
		case 'CB TEL':
		case 'TPE':
			return (D_K_BANK_CB ?D_K_BANK_CB :static::D_K_BANK_DEFAULT);
		case 'Chèque':
		case 'Cheque':
		case 'Transfert bancaire':
			return (D_K_BANK_DEFAULT ?D_K_BANK_DEFAULT :static::D_K_BANK_DEFAULT);
		case 'Espèces':
		case 'Especes':
		case 'Caisse':
			return (D_K_BANK_CAISSE ?D_K_BANK_CAISSE :static::D_K_BANK_DEFAULT);
		case 'PrestaShop Checkout':
			return (D_K_BANK_DEFAULT ?D_K_BANK_DEFAULT :static::D_K_BANK_DEFAULT);
		default :
			return (D_K_BANK_DEFAULT ?D_K_BANK_DEFAULT :static::D_K_BANK_DEFAULT);
	}
}

}

sync_payment_pd::__init();
