<?php

class sync_product_pd extends sync_product
{
	// -- CHECK --

	// No cascade delete
	const PD_DELETE = false;
	
	public static function pd_check_sync(&$o, &$p_data)
	{
		// Nouveau produit sans référence ou "copy of" => on synchronise pas
		if (
			empty($o['d_oid'])
			&& (
				(empty($p_data['product']['reference']) && empty($p_data['product_attribute']))
				|| (empty($p_data['product_attribute']['reference']) && !empty($p_data['product_attribute']))
				|| (substr($p_data['product_lang']['name'], 0, 7) == 'copy of') // Disable new duplicated products
				|| (substr($p_data['product_lang']['name'], 0, 6) == 'NOSYNC') // Temporary ref
			)
		)
			return false;
		
		// @todo check doublon + sendmail ?
	
		return true;
	}

// -- ACTIONS --

public function pd_execute_osync($otype, $oid)
{
	// Si produit maître avec déclinaisons, on synchronise toutes les déclinaisons
	if ($otype=='product' && static::p_product_check_combi($oid)) {
		$this->pd_osync_decl($oid);
		return;
	}
	
	return parent::pd_execute_osync($otype, $oid);
}

public function pd_execute_create($otype, $oid)
{
	// Si produit maître avec déclinaisons, on synchronise toutes les déclinaisons
	if ($otype=='product' && static::p_product_check_combi($oid)) {
		$this->pd_osync_decl($oid);
		return;
	}
	
	return parent::pd_execute_create($otype, $oid);
}

// -- SUB ACTIONS --

public function pd_delete_more(&$o)
{
	DB::d_update_row('product_extrafields', ['sync'=>0], ['fk_object'=>$o['d_oid']]);
}

public function pd_insert_more(&$o, &$d_data_map, &$p_data)
{
	// Un seul cas de $otype/$d_tref dans Doli
	$id = $o['d_oid'];

	//var_dump($p_data);

	// stock_available
	if (!empty($p_data['stock_available']))
		static::_pd_map_fk('stock', 'stock_available', $p_data['stock_available']['id_stock_available']);

	// products_dlc_dluo
	if (!empty($p_data['products_dlc_dluo'])) foreach($p_data['products_dlc_dluo'] as $row)
		static::_pd_map_fk('product_lot', 'products_dlc_dluo', $row['id']);
	
}

public function pd_update_more(&$o, &$d_data_map, &$d_data_map_create, &$d_data, &$p_data)
{
	$id = $o['d_oid'];
	$p_id = $p_data['product']['id_product'];
	$pa_id = (!empty($p_data['product_attribute']) ?$p_data['product_attribute']['id_product_attribute'] :0);

	// Set Prestashop reference from Dolibarr
	if ($pa_id && (empty($p_data['product_attribute']['reference']) || $p_data['product_attribute']['reference']=='NEW')) {
		// Si réf déjà exisatante on récup
		if (!empty($d_data['product']['ref']))
			$ref = $d_data['product']['ref'];
		else
			$ref = $d_data_map_create['product']['ref'];
		$sql = 'UPDATE '.DB_P_PREFIX.'product_attribute
			SET reference="'.$ref.'"
			WHERE id_product_attribute='.$pa_id;
		DB::p_update($sql);
	}
	elseif (empty($pa_id) && (empty($p_data['product']['reference']) || $p_data['product']['reference']=='NEW')) {
		// Si réf déjà exisatante on récup
		if (!empty($d_data['product']['ref']))
			$ref = $d_data['product']['ref'];
		else
			$ref = $d_data_map_create['product']['ref'];
		$sql = 'UPDATE '.DB_P_PREFIX.'product
			SET reference="'.$ref.'"
			WHERE id_product='.$p_id;
		DB::p_update($sql);
	}
	
	// Adds row in product_price if updated
	if(isset($d_data['product']) && (
		($d_data['product']['price'] != $d_data_map['product']['price'])
		|| ($d_data['product']['tva_tx'] != $d_data_map['product']['tva_tx'])
	)) {
		$row = [
			//'rowid' => ,
			'entity' => 1,
			'tms' => static::$datetime,
			'fk_product' => $id,
			'date_price' => static::$datetime,
			'price_level' => 1,
			'price' => $d_data_map['product']['price'],
			'price_ttc' => $d_data_map['product']['price_ttc'],
			'price_min' => $d_data['product']['price_min'],
			'price_min_ttc' => $d_data['product']['price_min_ttc'],
			'price_base_type' => 'HT',
			'default_vat_code' => NULL,
			'tva_tx' => $d_data_map['product']['tva_tx'],
			'recuperableonly' => 0,
			'localtax1_tx' => 0,
			'localtax1_type' => 0,
			'localtax2_tx' => 0,
			'localtax2_type' => 0,
			'fk_user_author' => static::D_K_USER,
			'tosell' => $d_data['product']['tosell'],
			'price_by_qty' => 0,
			'fk_price_expression' => NULL,
			'import_key' => NULL,
			'fk_multicurrency' => NULL,
			'multicurrency_code' => NULL,
			'multicurrency_tx' => 1,
			'multicurrency_price' => NULL,
			'multicurrency_price_ttc' => NULL,
		];
		var_dump($row);
		$r = DB::d_insert_row('product_price', $row);
	}

	// product_extrafields
	static::d_update_row('product_extrafields', ['fk_object'=>$id], $d_data_map, $d_data_map_create, $d_data);
	
	// déclinaison => pas de modif du kit
	if (!empty($pa_id)) {
		// NADA
	}
	// pack mappé par Presta
	elseif (!empty($d_data_map['product_association'])) {
		// analyse old assoc
		$d_assoc_old = [];
		if (isset($d_data['product_association'])) {
			foreach($d_data['product_association'] as &$row)
				$d_assoc_old[$row['fk_product_fils']] = $row;
		}
		$d_assoc_update = [];
		$d_assoc_stayed = [];
		$d_assoc_insert = [];
		$d_assoc_delete = [];
		// analyse new assoc => look into new and updated
		foreach($d_data_map['product_association'] as $row) {
			// ajout id père
			$row['fk_product_pere'] = $id;
			// Nouveau
			if (empty($d_assoc_old[$row['fk_product_fils']]))
				$d_assoc_insert[] = $row;
			// Déjà mais modif qte
			elseif ($d_assoc_old[$row['fk_product_fils']]['qty'] != $row['qty'])
				$d_assoc_update[$row['fk_product_fils']] = $row;
			// déjà identique
			else
				$d_assoc_stayed[$row['fk_product_fils']] = $row;
		}
		// Remove deleted
		foreach($d_assoc_old as $k=>$row) {
			// Pas déjà
			if (!isset($d_assoc_update[$k]) && !isset($d_assoc_stayed[$k]))
				$d_assoc_delete[] = $row['rowid'];
		}
		
		// DB actions
		foreach($d_assoc_insert as $row)
			DB::d_insert_row('product_association', $row);
		foreach($d_assoc_update as $k=>$row)
			DB::d_update_row('product_association', $row, ['rowid'=>$d_assoc_old[$k]['rowid']], $d_assoc_old[$k]);
		if (!empty($d_assoc_delete))
			DB::d_delete('DELETE FROM `'.DB_D_PREFIX.'product_association` WHERE rowid IN ('.implode(', ', $d_assoc_delete).')');
		
		// Après les modifs de kit sinon ça fait de la merde !!!!!! Putain de connerie qui m'a pris 1/2 journée
		if (DEBUG_AFF)
			var_dump($d_assoc_insert, $d_assoc_update, $d_assoc_stayed, $d_assoc_delete);
		// @todo check selon si on est dans mmiproductapi ou mmiproductdluoapi
		// Voir si on peut fusionner... pas évident du tout si on a des kits hétérogènes...
		// Dans doli il faut cascade le stock ssi pas ddm et ajouter contenu (éclater) ssi ddm
		if (!empty($d_assoc_insert) || !empty($d_assoc_update) || !empty($d_assoc_delete))
			static::_d_api('GET', 'mmiproductdluoapi/kit_stock_calculate/'.$id);
	}
	// Pas d'infos, on vire tout ce qu'on a
	elseif(!empty($d_data['product_association'])) {
		// Suppression de tout, puisque on a rien en entrée
		DB::d_delete('DELETE FROM `'.DB_D_PREFIX.'product_association` WHERE `fk_product_pere`='.$id);
	}

	// Image
	if (!empty($p_data['image']))
		$this->image_sync($p_data, $p_data['image']['id_image']);

	// product_supplier
	// Only add/update if exists, not a complete sync with deletion in Doli
	if (defined('PD_SYNC_SUPPLIER_PRICE') && PD_SYNC_SUPPLIER_PRICE && !empty($p_data['product_supplier'])) foreach($p_data['product_supplier'] as $row)
		static::_action('supplier_price', 'pd', 'osync', 'product_supplier', $row['id_product_supplier']);
	
	// Déclinaisons
	if (empty($pa_id))
		$this->pd_osync_decl($p_id);
}

public function pd_osync_decl($oid)
{
	$ok = false;
	$sql = 'SELECT p.id_product_attribute
		FROM '.DB_P_PREFIX.'product_attribute p
		WHERE p.id_product='.$oid;
	$q = DB::p_select($sql);
	$ok = false;
	while(list($pa_id)=$q->fetch_row()) {
		if (static::_action('product', 'pd', 'osync', 'product_attribute', $pa_id))
			$ok = true;
	}
	if ($ok) {
		echo 'updatep';
		static::_notif_ts('p', 'product', $oid);
	}
	return;
}

public function image_sync(&$p_data, $i_id)
{
	$p_filename = $i_id.'.jpg';
	$p_folder = P_FOLDER_ROOT.'/img/p';

	$n = strlen($i_id);
	for($i=0; $i<$n; $i++)
		$p_folder .= '/'.substr($i_id, $i, 1);
	
	$p_f = $p_folder.'/'.$p_filename;
	if (DEBUG_AFF)
		var_dump($p_f);

	if (file_exists($p_f)) {
		//var_dump($p_f);
		if (!empty($p_data['product_attribute']))
			$ref = $p_data['product_attribute']['reference'];
		else
			$ref = $p_data['product']['reference'];
		if (empty($ref))
			return;

		$d_filename = $ref.'-'.$i_id.'.jpg';
		$d_folder = D_FOLDER_ROOT.'/documents/produit/'.$ref;
		if (!file_exists($d_folder))
			mkdir($d_folder);
		$d_f = $d_folder.'/'.$d_filename;
		if (DEBUG_AFF)
			var_dump($d_f);
		exec("rsync -a \"$p_f\" \"$d_f\"");

		// Suppression ancienne image
		$fp = opendir($d_folder);
		while($filename=readdir($fp)) {
			if (!in_array($filename, ['.', '..', $d_filename])) {
				if (DEBUG_AFF)
					var_dump('DELETE : '.$filename);
				unlink($d_folder.'/'.$filename);
			}
		}
		return true;
	}
}

public function p_url($id_product, $id_product_attribute, $link_rewrite, $options=[])
{
	return P_WEBSITE.(P_URL_PREFIX_LANG ?'/'.P_LANG :'').(P_URL_PREFIX_CAT ?'/'.$options['cat_rewrite'] :'/p').'/'.$id_product.(P_URL_ATTRIBUTE_ID && $id_product_attribute ?'-'.$id_product_attribute :'').'-'.$link_rewrite.(P_URL_SUFFIX_HTML ?'.html' :'');
}

// -- MAPPING --

public function pd_map_update(&$o, &$p_data, &$d_data=[])
{
	if ($o['p_tref']=='product_attribute')
		return $this->pd_map_update_combi($o, $p_data, $d_data);
	else
		return $this->pd_map_update_simple($o, $p_data, $d_data);
}
public function pd_map_update_common(&$o, &$p_data, &$d_data=[])
{
}
public function pd_map_update_simple(&$o, &$p_data, &$d_data=[])
{
	$tva_tx = static::_pd_map_tva_taux($p_data);
	$tms = max($p_data['product']['date_upd'], (isset($d_data['product']['tms']) ?$d_data['product']['tms'] :0));

	// product_supplier
	if (!empty($p_data['product_supplier'])) foreach($p_data['product_supplier'] as $row) {
		if ($p_data['product']['id_supplier']==$row['id_supplier'])
			$supplier_ref = $row['product_supplier_reference'];
	}

	static::_d_units_load();

	$url = $this->p_url($p_data['product']['id_product'], 0, $p_data['product_lang']['link_rewrite'], P_URL_PREFIX_CAT ?['cat_rewrite'=>$p_data['category_lang']['link_rewrite']] :[]);

	$d_data_map = [
		'product' => [
			//'label' => $p_data['product_lang']['name'],
			//'description' => static::_description_format($p_data),
			'tms' => $tms,
			'cost_price' => $p_data['product_shop']['wholesale_price'],
			'price' => $p_data['product_shop']['price'],
			'price_ttc' => round($p_data['product_shop']['price']*(1+$tva_tx/100), 5),
			'tva_tx' => $tva_tx,
			'weight' => PD_UNIT_UPDATE || empty($d_data['product']['weight']) ?$p_data['product']['weight'] :$d_data['product']['weight'],
			'weight_units' => PD_UNIT_UPDATE || empty($d_data['product']['weight_units']) ?static::$_d_units_code['KG'] :$d_data['product']['weight_units'],
			'length' => PD_UNIT_UPDATE || empty($d_data['product']['length']) ?$p_data['product']['depth'] :$d_data['product']['length'],
			'length_units' => PD_UNIT_UPDATE || empty($d_data['product']['length_units']) ?static::$_d_units_code['CM'] :$d_data['product']['length_units'],
			'width' => PD_UNIT_UPDATE || empty($d_data['product']['width']) ?$p_data['product']['width'] :$d_data['product']['width'],
			'width_units' => PD_UNIT_UPDATE || empty($d_data['product']['width_units']) ?static::$_d_units_code['CM'] :$d_data['product']['width_units'],
			'height' => PD_UNIT_UPDATE || empty($d_data['product']['height']) ?$p_data['product']['height'] :$d_data['product']['height'],
			'height_units' => PD_UNIT_UPDATE || empty($d_data['product']['height_units']) ?static::$_d_units_code['CM'] :$d_data['product']['height_units'],
			'barcode' => static::_pd_map_barcode($p_data),
			'fk_barcode_type' => static::_pd_map_barcode_type($p_data),
			'url' => $url,
		],
		'product_extrafields' => [
			'p_active' => $p_data['product']['active'],
			'p_available_for_order' => $p_data['product']['available_for_order'],
			'p_online_only' => $p_data['product']['online_only'],
			'garantie' => (isset($p_data['garantie']) ?substr(strip_tags($p_data['garantie']['content']), 0, 255) :''),
			'fk_soc_fournisseur' => static::_pd_map_fk('supplier', 'supplier', $p_data['product']['id_supplier']),
			'supplier_ref' => (!empty($supplier_ref) ?$supplier_ref :NULL),
			'compose' => (!empty($p_data['pack']) ?1 :NULL),
			//'longdescript' => $p_data['product_lang']['description'],
			'fk_categorie' => static::_pd_map_fk('cat', 'category', $p_data['product']['id_category_default']),
		],
	];

	// Active
	if (defined('PRODUCT_ACTIVE_SYNC') && PRODUCT_ACTIVE_SYNC) {
		if (PRODUCT_ACTIVE_SYNC===true || in_array(PRODUCT_ACTIVE_SYNC, ['all', 'sell']))
			$d_data_map['product']['tosell'] = (($p_data['product']['active'] && (!PRODUCT_FIELD_DECLI_DISABLED || empty($p_data['tot_switch_attribute_disabled']))) ?1 :0);
		if (PRODUCT_ACTIVE_SYNC===true || in_array(PRODUCT_ACTIVE_SYNC, ['all', 'buy']))
			$d_data_map['product']['tobuy'] = (($p_data['product']['active'] && (!PRODUCT_FIELD_DECLI_DISABLED || empty($p_data['tot_switch_attribute_disabled']))) ?1 :0);
	}
	
	// Label & Desc No Sync
	if (!defined('PRODUCT_FIELD_LABEL_NOSYNC') || !PRODUCT_FIELD_LABEL_NOSYNC)
		$d_data_map['product']['label'] = $p_data['product_lang']['name'];
	if (!defined('PRODUCT_FIELD_DESC_NOSYNC') || !PRODUCT_FIELD_DESC_NOSYNC)
		$d_data_map['product']['description'] = static::_description_format($p_data);

	// Image de référence
	if (!empty($p_data['image']))
		$d_data_map['product_extrafields']['p_image'] = P_WEBSITE.'/'.$p_data['image']['id_image'].'-large_default/'.$p_data['product_lang']['link_rewrite'].'.jpg';

	// Référence non nulle !
	if (!empty($p_data['product']['reference']) && $p_data['product']['reference']!='NEW')
		$d_data_map['product']['ref'] = $p_data['product']['reference'];

	if (PUBLIC_PRICE) {
		$d_data_map['product_extrafields']['public_price'] = (!empty($p_data['product']['recommended_public_price']) ?round($p_data['product']['recommended_public_price'], 2) :NULL);
	}

	if (ECOTAX) {
		$d_data_map['product_extrafields']['ecotaxdeee'] = ((!empty($p_data['product']['ecotax']) && $p_data['product']['ecotax']>0) ?$p_data['product']['ecotax'] :NULL);
	}
	
	if (PRODUCT_FIELD_CUSTOM && array_key_exists('sur_mesure', $p_data['product'])) {
		$d_data_map['product_extrafields']['custom'] = $p_data['product']['sur_mesure'] ?1 :0;
		//$p_map_data['product']['sur_mesure_available'] = $d_data['product_extrafields']['custom'];
		//$p_map_data['product']['sur_mesure_quotation'] = $d_data['product_extrafields']['custom'];
	}
	if (PRODUCT_FIELD_SUPPLIER_DIRECT_DELIVERY && array_key_exists('supplier_direct_delivery', $p_data['product'])) {
		$d_data_map['product_extrafields']['supplier_direct_delivery'] = $p_data['product']['supplier_direct_delivery'] ?1 :0;
		//$p_map_data['product']['supplier_direct_delivery_available'] = $d_data['product_extrafields']['supplier_direct_delivery'];
	}

	// Pack
	$d_data_map['product_association'] = [];
	if (isset($p_data['pack'])) {
		foreach($p_data['pack'] as $row) {
			if ($row['id_product_attribute_item'])
				$fk_product_fils = static::_pd_map_fk('product', 'product_attribute', $row['id_product_attribute_item']);
			else
				$fk_product_fils = static::_pd_map_fk('product', 'product', $row['id_product_item']);
			if (!empty($fk_product_fils)) {
				$d_data_map['product_association'][] = [
					//'fk_product_pere' => $o['d_oid'],
					'fk_product_fils' => $fk_product_fils,
					'qty' => $row['quantity'],
					'incdec' => (PRODUCT_LOT ?0 :1), // On gère la décrémentation du stock
				];
			}
		}
	}
	
	return $d_data_map;
}
public function pd_map_update_combi(&$o, &$p_data, &$d_data=[])
{	
	$tva_tx = static::_pd_map_tva_taux($p_data);
	$tms = max($p_data['product_attribute']['date_upd'], $p_data['product']['date_upd'], (isset($d_data['product']['tms']) ?$d_data['product']['tms'] :0));

	// product_supplier
	if (!empty($p_data['product_supplier'])) foreach($p_data['product_supplier'] as $row) {
		if ($p_data['product']['id_supplier']==$row['id_supplier'])
			$supplier_ref = $row['product_supplier_reference'];
	}
	
	static::_d_units_load();

	$url = $this->p_url($p_data['product']['id_product'], $p_data['product_attribute']['id_product_attribute'], $p_data['product_lang']['link_rewrite'], P_URL_PREFIX_CAT ?['cat_rewrite'=>$p_data['category_lang']['link_rewrite']] :[]);

	$d_data_map = [
		'product' => [
			//'label' => $p_data['product_lang']['name'].' - '.$attribute_names,
			//'description' => static::_description_format($p_data),
			'tms' => $tms,
			'cost_price' => $p_data['product_attribute_shop']['wholesale_price'],
			'price' => ($p_data['product_shop']['price']+$p_data['product_attribute_shop']['price']),
			'price_ttc' => round(($p_data['product_shop']['price']+$p_data['product_attribute_shop']['price'])*(1+$tva_tx/100), 5),
			'tva_tx' => $tva_tx,
			'weight' => PD_UNIT_UPDATE || empty($d_data['product']['weight']) ?round($p_data['product']['weight']+$p_data['product_attribute']['weight'], 5) :$d_data['product']['weight'],
			'weight_units' => PD_UNIT_UPDATE || empty($d_data['product']['weight_units']) ?static::$_d_units_code['KG'] :$d_data['product']['weight_units'],
			'length' => PD_UNIT_UPDATE || empty($d_data['product']['length']) ?(!empty($p_data['cdcd_product_combination_dimensions']['depth']) ?$p_data['cdcd_product_combination_dimensions']['depth'] :$p_data['product']['depth']) :$d_data['product']['length'],
			'length_units' => PD_UNIT_UPDATE || empty($d_data['product']['length_units']) ?static::$_d_units_code['CM'] :$d_data['product']['length_units'],
			'width' => PD_UNIT_UPDATE || empty($d_data['product']['width']) ?(!empty($p_data['cdcd_product_combination_dimensions']['width']) ?$p_data['cdcd_product_combination_dimensions']['width'] :$p_data['product']['width']) :$d_data['product']['width'],
			'width_units' => PD_UNIT_UPDATE || empty($d_data['product']['width_units']) ?static::$_d_units_code['CM'] :$d_data['product']['width_units'],
			'height' => PD_UNIT_UPDATE || empty($d_data['product']['height']) ?(!empty($p_data['cdcd_product_combination_dimensions']['height']) ?$p_data['cdcd_product_combination_dimensions']['height'] :$p_data['product']['height']) :$d_data['product']['height'],
			'height_units' => PD_UNIT_UPDATE || empty($d_data['product']['height_units']) ?static::$_d_units_code['CM'] :$d_data['product']['height_units'],
			'barcode' => static::_pd_map_barcode($p_data),
			'fk_barcode_type' => static::_pd_map_barcode_type($p_data),
			'url' => $url,
		],
		'product_extrafields' => [
			'p_active' => $p_data['product']['active'],
			'p_available_for_order' => $p_data['product']['available_for_order'],
			'p_online_only' => $p_data['product']['online_only'],
			'garantie' => (isset($p_data['garantie']) ?substr(strip_tags($p_data['garantie']['content']), 0, 255) :''),
			'fk_soc_fournisseur' => static::_pd_map_fk('supplier', 'supplier', $p_data['product']['id_supplier']),
			'supplier_ref' => (!empty($supplier_ref) ?$supplier_ref :NULL),
			//'compose' => (!empty($p_data['pack']) ?1 :NULL), // Pas de modif
			'longdescript' => $p_data['product_lang']['description'],
			'fk_categorie' => static::_pd_map_fk('cat', 'category', $p_data['product']['id_category_default']),
		],
	];

	// Active
	if (defined('PRODUCT_ACTIVE_SYNC') && PRODUCT_ACTIVE_SYNC) {
		if (in_array(PRODUCT_ACTIVE_SYNC, [true, 'all', 'sell']))
			$d_data_map['product']['tosell'] = (($p_data['product']['active'] && (!PRODUCT_FIELD_DECLI_DISABLED || empty($p_data['tot_switch_attribute_disabled']))) ?1 :0);
		if (in_array(PRODUCT_ACTIVE_SYNC, [true, 'all', 'buy']))
			$d_data_map['product']['tobuy'] = (($p_data['product']['active'] && (!PRODUCT_FIELD_DECLI_DISABLED || empty($p_data['tot_switch_attribute_disabled']))) ?1 :0);
	}

	// Label & Desc No Sync
	if (!defined('PRODUCT_FIELD_LABEL_NOSYNC') || !PRODUCT_FIELD_LABEL_NOSYNC) {
		$this->_attributes_load();
		$attributes = [];
		foreach($p_data['product_attribute_combination'] as $row)
			$attributes[] = static::$_p_attribute[$row['id_attribute']];
		$attribute_names = implode(', ', $attributes);
		$d_data_map['product']['label'] = $p_data['product_lang']['name'].' - '.$attribute_names;
	}
	if (!defined('PRODUCT_FIELD_DESC_NOSYNC') || !PRODUCT_FIELD_DESC_NOSYNC)
		$d_data_map['product']['description'] = static::_description_format($p_data);

	// Image de référence
	if (!empty($p_data['image']))
		$d_data_map['product_extrafields']['p_image'] = P_WEBSITE.'/'.$p_data['image']['id_image'].'-large_default/'.$p_data['product_lang']['link_rewrite'].'.jpg';

	// Référence non nulle !
	if (!empty($p_data['product_attribute']['reference']) && $p_data['product_attribute']['reference']!='NEW')
		$d_data_map['product']['ref'] = $p_data['product_attribute']['reference'];

	if (PUBLIC_PRICE) {
		$d_data_map['product_extrafields']['public_price'] = (!empty($p_data['product_attribute']['recommended_public_price']) ?round($p_data['product_attribute']['recommended_public_price'], 2) :NULL);
	}

	if (ECOTAX) {
		// Fix bug 0.000 not empty
		$d_data_map['product_extrafields']['ecotaxdeee'] = ((!empty($p_data['product_attribute']['ecotax']) && $p_data['product_attribute']['ecotax']>0) ?$p_data['product_attribute']['ecotax'] :NULL);
	}

	if (PRODUCT_FIELD_CUSTOM && array_key_exists('sur_mesure', $p_data['product'])) {
		$d_data_map['product_extrafields']['custom'] = $p_data['product']['sur_mesure'] ?1 :0;
		//$p_map_data['product']['sur_mesure_available'] = $d_data['product_extrafields']['custom'];
		//$p_map_data['product']['sur_mesure_quotation'] = $d_data['product_extrafields']['custom'];
	}
	if (PRODUCT_FIELD_SUPPLIER_DIRECT_DELIVERY && array_key_exists('supplier_direct_delivery', $p_data['product'])) {
		$d_data_map['product_extrafields']['supplier_direct_delivery'] = $p_data['product']['supplier_direct_delivery'] ?1 :0;
		//$p_map_data['product']['supplier_direct_delivery_available'] = $d_data['product_extrafields']['supplier_direct_delivery'];
	}

	if (PRODUCT_FIELD_DECLI_DISABLED)
		$d_data_map['product_extrafields']['p_decli_disabled'] = !empty($p_data['tot_switch_attribute_disabled']) ?1 :NULL;

	return $d_data_map;
}

public function pd_map_create(&$o, &$p_data)
{
	//var_dump($o);
	$o['d_tref'] = static::D_TABLE_MAIN;

	if ($o['p_tref'] == 'product')
		return $this->pd_map_create_simple($o, $p_data);
	elseif ($o['p_tref'] == 'product_attribute')
		return $this->pd_map_create_combi($o, $p_data);
}
public function pd_map_create_common(&$o, &$p_data)
{
}
public function pd_map_create_simple(&$o, &$p_data)
{
	// Le produit doit être enregistré proprement dans Prestashop
	if (empty($p_data['product']['state']))
		return;

	// On doit spécifier qu'on veut créer la réf
	if (empty($p_data['product']['reference']))
		return;

	static::_d_units_load();

	$ref = ($p_data['product']['reference']!='NEW' ?$p_data['product']['reference'] :static::d_ref());
	//var_dump($ref); die();

	$d_data_map = [
		'product' => [
			//'rowid' => '',
			'ref' => $ref,
			'entity' => 1,
			'ref_ext' => NULL,
			'datec' => $p_data['product']['date_add'],
			'tms' => $p_data['product']['date_upd'],
			'fk_parent' => 0, // @todo voir si combi !
			'label' => $p_data['product_lang']['name'],
			'description' => $p_data['product_lang']['description_short'],
			'note_public' => NULL,
			'note' => NULL,
			'customcode' => NULL,
			'fk_country' => NULL,
			//'price' => $p_data['product']['price'],
			//'price_ttc' => $p_data['product']['price'],
			'price_min' => '0',
			'price_min_ttc' => '0',
			'price_base_type' => 'HT',
			'cost_price' => $p_data['product_shop']['wholesale_price'],
			'default_vat_code' => NULL,
			//'tva_tx' => static::_pd_map_tva_taux($p_data), // update
			'recuperableonly' => '0',
			'localtax1_tx' => '0',
			'localtax1_type' => '0',
			'localtax2_tx' => '0',
			'localtax2_type' => '0',
			'fk_user_author' => static::D_K_USER,
			'fk_user_modif' => static::D_K_USER,
			'tosell' => 1,
			'tobuy' => 1,
			'onportal' => 0,
			'tobatch' => static::_pd_map_batch($p_data), // @todo 1 si dlc
			'fk_product_type' => 0, // Produit par défaut
			'duration' => NULL,
			'seuil_stock_alerte' => defined('PRODUCT_SEUIL_STOCK_ALERTE') ?PRODUCT_SEUIL_STOCK_ALERTE :NULL,
			'url' => '',
			'barcode' => static::_pd_map_barcode($p_data),
			'fk_barcode_type' => static::_pd_map_barcode_type($p_data),
			'accountancy_code_sell' => defined('PRODUCT_ACCOUNTANCY_CODE') ?PRODUCT_ACCOUNTANCY_CODE :NULL,
			'accountancy_code_sell_intra' => NULL,
			'accountancy_code_sell_export' => NULL,
			'accountancy_code_buy' => NULL,
			'accountancy_code_buy_intra' => NULL,
			'accountancy_code_buy_export' => NULL,
			'partnumber' => NULL,
			'net_measure' => NULL,
			'net_measure_units' => NULL,
			'weight' => $p_data['product']['weight'],
			'weight_units' => static::$_d_units_code['KG'],
			'length' => $p_data['product']['depth'],
			'length_units' => static::$_d_units_code['CM'],
			'width' => $p_data['product']['width'],
			'width_units' => static::$_d_units_code['CM'],
			'height' => $p_data['product']['height'],
			'height_units' => static::$_d_units_code['CM'],
			'surface' => NULL,
			'surface_units' => NULL,
			'volume' => NULL,
			'volume_units' => NULL,
			'stock' => $p_data['stock_available']['quantity'],
			'pmp' => '0',
			'fifo' => NULL,
			'lifo' => NULL,
			'fk_default_warehouse' => static::D_K_WAREHOUSE,
			'canvas' => NULL,
			'finished' => NULL,
			'hidden' => '0',
			'import_key' => NULL,
			'model_pdf' => NULL,
			'fk_price_expression' => NULL,
			'desiredstock' => defined('PRODUCT_DESIRED_STOCK') ?PRODUCT_DESIRED_STOCK :NULL,
			'fk_unit' => NULL,
			'price_autogen' => '0',
			'fk_project' => NULL,
			'fk_state' => NULL,
			'lifetime' => NULL,
			'qc_frequency' => NULL,
			'batch_mask' => NULL,
		],
		'product_extrafields' => [
			//'rowid' => '',
			'tms' => static::$datetime,
			//'fk_object' => '',
			'import_key' => NULL,
			'sync' => 1,
		],
	];

	return $d_data_map;
}

public function pd_map_create_combi(&$o, &$p_data)
{
	// Le produit doit être enregistré proprement dans Prestashop
	if (empty($p_data['product']['state']))
		return;

	// On doit spécifier qu'on veut créer la réf
	if (empty($p_data['product_attribute']['reference']))
		return;
	
	static::_d_units_load();
	
	// Label with attributes
	$this->_attributes_load();
	$attributes = [];
	foreach($p_data['product_attribute_combination'] as $row)
		$attributes[] = static::$_p_attribute[$row['id_attribute']];
	$attribute_names = implode(', ', $attributes);

	$ref = ($p_data['product_attribute']['reference']!='NEW' ?$p_data['product_attribute']['reference'] :static::d_ref());

	$d_data_map = [
		'product' => [
			//'rowid' => '',
			'ref' => $ref,
			'entity' => 1,
			'ref_ext' => NULL,
			'datec' => max($p_data['product_attribute']['date_upd'], $p_data['product']['date_add']),
			'tms' => max($p_data['product_attribute']['date_upd'], $p_data['product']['date_upd']),
			'fk_parent' => 0, // @todo voir si combi !
			'label' => $p_data['product_lang']['name'].' - '.$attribute_names,
			'description' => $p_data['product_lang']['description_short'],
			'note_public' => NULL,
			'note' => NULL,
			'customcode' => NULL,
			'fk_country' => NULL,
			'price' => $p_data['product_shop']['price']+$p_data['product_attribute_shop']['price'],
			'price_ttc' => $p_data['product_shop']['price']+$p_data['product_attribute_shop']['price'],
			'price_min' => '0',
			'price_min_ttc' => '0',
			'price_base_type' => 'HT',
			'cost_price' => $p_data['product_attribute_shop']['wholesale_price'],
			'default_vat_code' => NULL,
			'tva_tx' => static::_pd_map_tva_taux($p_data),
			'recuperableonly' => '0',
			'localtax1_tx' => '0',
			'localtax1_type' => '0',
			'localtax2_tx' => '0',
			'localtax2_type' => '0',
			'fk_user_author' => static::D_K_USER,
			'fk_user_modif' => static::D_K_USER,
			'tosell' => 1,
			'tobuy' => 1,
			'onportal' => 0,
			'tobatch' => static::_pd_map_batch($p_data), // @todo 1 si dlc
			'fk_product_type' => 0, // ?
			'duration' => NULL,
			'seuil_stock_alerte' => defined('PRODUCT_SEUIL_STOCK_ALERTE') ?PRODUCT_SEUIL_STOCK_ALERTE :NULL,
			'url' => '',
			'barcode' => static::_pd_map_barcode($p_data),
			'fk_barcode_type' => static::_pd_map_barcode_type($p_data),
			'accountancy_code_sell' => defined('PRODUCT_ACCOUNTANCY_CODE') ?PRODUCT_ACCOUNTANCY_CODE :NULL,
			'accountancy_code_sell_intra' => NULL,
			'accountancy_code_sell_export' => NULL,
			'accountancy_code_buy' => NULL,
			'accountancy_code_buy_intra' => NULL,
			'accountancy_code_buy_export' => NULL,
			'partnumber' => NULL,
			'net_measure' => NULL,
			'net_measure_units' => NULL,
			'weight' => $p_data['product']['weight']+$p_data['product_attribute']['weight'],
			'weight_units' => static::$_d_units_code['KG'],
			'length' => $p_data['product']['depth'],
			'length_units' => static::$_d_units_code['CM'],
			'width' => $p_data['product']['width'],
			'width_units' => static::$_d_units_code['CM'],
			'height' => $p_data['product']['height'],
			'height_units' => static::$_d_units_code['CM'],
			'surface' => NULL,
			'surface_units' => NULL,
			'volume' => NULL,
			'volume_units' => NULL,
			'stock' => $p_data['stock_available']['quantity'],
			'pmp' => '0',
			'fifo' => NULL,
			'lifo' => NULL,
			'fk_default_warehouse' => static::D_K_WAREHOUSE,
			'canvas' => NULL,
			'finished' => NULL,
			'hidden' => '0',
			'import_key' => NULL,
			'model_pdf' => NULL,
			'fk_price_expression' => NULL,
			'desiredstock' => defined('PRODUCT_DESIRED_STOCK') ?PRODUCT_DESIRED_STOCK :NULL,
			'fk_unit' => NULL,
			'price_autogen' => '0',
			'fk_project' => NULL,
			'fk_state' => NULL,
			'lifetime' => NULL,
			'qc_frequency' => NULL,
			'batch_mask' => NULL,
		],
		'product_extrafields' => [
			//'rowid' => '',
			'tms' => static::$datetime,
			//'fk_object' => '',
			'import_key' => NULL,
			'sync' => 1,
		],
	];

	return $d_data_map;
}

// -----------

// Field MAPPING

public static function _pd_map_batch(&$orig_data, &$dest_tree=[])
{
	// @todo vérif si besoin
	return ((PRODUCT_LOT && empty($orig_data['pack'])) || !empty($orig_data['products_dlc_dluo']) ?1 :0);
}

public static function _pd_map_tva_taux(&$orig_data, &$dest_tree=[])
{
	// @todo mapper les taxes proprement !!
	switch($orig_data['product']['id_tax_rules_group']) {
		case 1:
			return 20;
		case 2:
			return 10;
		case 3:
			return 5.5;
		case 4:
			return 2.1;
		case 5:
			return 0;
	}
}

public static function _pd_map_barcode(&$orig_data, &$dest_tree=[])
{
	if (isset($orig_data['product_attribute'])) {
		if ($orig_data['product_attribute']['ean13'])
			return $orig_data['product_attribute']['ean13'];
		if ($orig_data['product_attribute']['upc'])
			return $orig_data['product_attribute']['upc'];
		if ($orig_data['product_attribute']['isbn'])
			return $orig_data['product_attribute']['isbn'];
	}
	else {
		if ($orig_data['product']['ean13'])
			return $orig_data['product']['ean13'];
		if ($orig_data['product']['upc'])
			return $orig_data['product']['upc'];
		if ($orig_data['product']['isbn'])
			return $orig_data['product']['isbn'];
	}
}
public static function _pd_map_barcode_type(&$orig_data, &$dest_tree=[])
{
	if (isset($orig_data['product_attribute'])) {
		if ($orig_data['product_attribute']['ean13'])
			return 2;
		if ($orig_data['product_attribute']['upc'])
			return 3;
		if ($orig_data['product_attribute']['isbn'])
			return 4;
	}
	else {
		if ($orig_data['product']['ean13'])
			return 2;
		if ($orig_data['product']['upc'])
			return 3;
		if ($orig_data['product']['isbn'])
			return 4;
	}
}

public static function _description_format(&$data)
{
	return str_replace(
		['<p>', '</p>', '<br/>', '<br />'],
		['', "<br />\r\n", "<br />\r\n", "<br />\r\n"],
		strip_tags($data['product_lang']['description_short'], '<p><br>')
			.(isset($data['garantie']) ?"<br />GARANTIE : ".strip_tags($data['garantie']['content'], '<p><br>') :'')
	);
}

}

sync_product_pd::__init();
